/* Interpolation routine(s) copied from gimp,
 * (c) 1995 Spencer Kimball and Peter Mattis
 *
 * (c) 2014 - 2023 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <https://www.gnu.org/licenses/>.
 */

/* TODO: Refactor code */

#include <math.h>
#include "tools.h"
#include "brushes.h"
#include "dibuja.h"
#include "util.h"
#include "draw-funcs.h"
#include "simple-canvas.h"
#include "undo.h"
#include "file.h"

#define MOTION_PAINT		1
#define SQR(x)				((x) * (x))
#define EPSILON				0.00001

#define BRUSH_WIDTH			(1.0)
#define BRUSH_HEIGHT		BRUSH_WIDTH
#define BRUSH_SPACING		(1.0)
#define IMG_WIDTH			DEF_BRUSH_DIM

#define darea_set_img(x,p)	g_object_set_data_full(G_OBJECT(x),("pixbuf"), \
											((gpointer)(p)), (g_object_unref))

typedef enum{
	BRUSH_PENCIL = 0,
	BRUSH_RECTANGLE,
	BRUSH_ELLIPSE,
	BRUSH_IMAGE,
	BRUSH_GRADIENT,
	BRUSH_CROSSHATCH,
	BRUSH_RANDDOTS,
	BRUSH_FSLASH,
	BRUSH_BSLASH,
	BRUSH_STAR,
	BRUSH_PENT,
	BRUSH_LAST
}BrushType;

typedef struct{
	BrushType type;
	double spacing;
	gboolean filled;
	gint size;
	char desc[20];
	GtkWidget *imgwidget;
}BrushInfo;

typedef struct _PaintCore PaintCore;
typedef void (*PaintFunc)   (PaintCore *, cairo_t *);

struct _PaintCore {
	double startx; //  starting x coord
	double starty; //  starting y coord

	double curx; //  current x coord
	double cury; //  current y coord

	double lastx; //  last x coord
	double lasty; //  last y coord

	int state; //  state of buttons and keys

	double distance; //  distance traveled by brush  
	double spacing; //  distance traveled by brush  

	int x1, y1;  //  image space coordinate
	int x2, y2; //  image space coords

	double width; // brush width
	double height; // brush height

 	PaintFunc paint_func; //  painting function
 	BrushInfo *binfo;
};

void rb_brush_toggled(GtkToggleButton *button);
static gboolean brush_button_press(GtkWidget *widget, GdkEventButton *event, gpointer data);
static gboolean brush_motion(GtkWidget *widget, GdkEventMotion *event, gpointer data);
static gboolean brush_button_release(GtkWidget *widget, GdkEventButton *event, gpointer data);
static gboolean brush_expose(GtkWidget *widget, cairo_t *cr, gpointer data);
static void brush_finish(void);
static void brush_set_type(BrushInfo *bi);

static int brush_set_current_brush(GtkImage *img);
static void brush_set_height(double height);
static SCColor fg = {0};
static SCColor bg = {0};
// Brush drawing funcs
static void draw_ellip(PaintCore *core, cairo_t *cr);
static void brush_draw_image(PaintCore *core, cairo_t *cr);
static void draw_gradient(PaintCore *core, cairo_t *cr);
static void draw_crosshatch(PaintCore *core, cairo_t *cr);

static DrawTool tool = {
				brush_button_press,
				brush_motion,
				brush_button_release,
				brush_expose,
				brush_finish,
				NULL,
				NULL
			};

PaintCore pc = {
				0, 0,
				0, 0,
				0, 0,
				0,
				BRUSH_SPACING, BRUSH_SPACING,
				0, 0, 0, 0,
				BRUSH_WIDTH, BRUSH_HEIGHT,
				(PaintFunc)draw_ellip,
				NULL
			};

static cairo_surface_t *undo_tmp_surf = NULL;
static BrushType brush_type = BRUSH_ELLIPSE;
static gboolean rotate = FALSE;


DrawTool *brush_get_tool(void)
{
	return &tool;
}

static void brush_set_update_rect(GdkRectangle *area)
{
	double z = dibuja_get_zoom();
	set_update_rect(area,
					(int)pc.curx, (int)pc.cury, (int)pc.lastx, (int)pc.lasty,
					z);

	resize_rect(area, pc.width * z + .5, pc.height * z + .5);
}

static GdkPixbuf *darea_get_img(GtkWidget *w)
{
	GdkPixbuf *pixbuf;

	pixbuf = GDK_PIXBUF(g_object_get_data(G_OBJECT(w), "pixbuf"));
	if(GDK_PIXBUF(pixbuf)) return pixbuf;

	return GDK_PIXBUF(g_object_get_data(G_OBJECT(w), "default-image"));
}

static void brush_image_darea_show(gboolean show)
{
	GtkWidget *darea = get_widget("img_brush_darea");

	if(!show){
		gtk_widget_set_size_request(darea, 1, 1);
		gtk_widget_hide(darea);
	}
	else{
		GdkPixbuf *pb;
		int size;

		pb = darea_get_img(darea);
		size = MAX(gdk_pixbuf_get_width(pb), gdk_pixbuf_get_height(pb));
		gtk_widget_set_size_request(darea, size, size);
		gtk_widget_show(darea);
	}
}

static void brush_finish(void)
{
	if(undo_tmp_surf){
		cairo_surface_destroy(undo_tmp_surf);
		undo_tmp_surf = NULL;
	}
	if(BRUSH_IMAGE != brush_type){
		brush_image_darea_show(FALSE);
	}
}

static void brush_brush_img_setup(GdkPixbuf *pixbuf)
{
	int size;

	g_return_if_fail(GDK_IS_PIXBUF(pixbuf)) ;

	size = MAX(gdk_pixbuf_get_width(pixbuf), gdk_pixbuf_get_height(pixbuf));

	brush_set_width((double) size);
	brush_set_height((double) size);
	util_spin_set_val("spinbutton_brush_spacing", (double) size);
	util_spin_set_val("spinbutton_brush_width", (double) size);
}

int brush_set_brush_image_from_selection(GdkPixbuf *pixbuf)
{
	GtkWidget *image, *button, *darea;
	int w, h;

	if(!GDK_IS_PIXBUF(pixbuf)) return 0;

	w = gdk_pixbuf_get_width(pixbuf);
	h = gdk_pixbuf_get_height(pixbuf);
	if((w > 100) || (h > 100)){
		message_box_ok(GTK_MESSAGE_ERROR, 
						_("The maximum size for image brushes\n"
						  "is 100 x 100 pixels, the current size is\n"
						  "%d x %d pixels."), w, h);
		return 0;
	}

	darea = get_widget("img_brush_darea");
	darea_set_img(darea, gdk_pixbuf_copy(pixbuf));

	button = get_widget("img-brush");
	image = gtk_bin_get_child(GTK_BIN(button));
	brush_set_current_brush(GTK_IMAGE(image));

	return 1;
}

void brush_image_brush_activate(void)
{
	gtk_toggle_button_set_active(
							GTK_TOGGLE_BUTTON(get_widget("img-brush")), TRUE);
}

void brush_set_width(double width)
{

	if(width >= 1.){
		pc.width = width;
	}
	else{
		pc.width = 1.;
	}

	if(BRUSH_IMAGE != brush_type)
		brush_set_height(pc.width);
}

static void brush_set_height(double height)
{

	if(height >= 1.){
		pc.height = height;
	}
	else{
		pc.height = height;
	}
}

void brush_set_spacing(double spacing)
{

	if(spacing >= 1.){
		pc.spacing = spacing;
	}
	else{
		pc.spacing = 1.;
	}
}
// TODO: Draw alternating images
static void brush_draw_image(PaintCore *core, cairo_t *cr)
{
	double cx, cy;
	GdkPixbuf *pixbuf;
	GtkWidget *darea;

	darea = get_widget("img_brush_darea");
	pixbuf = darea_get_img(darea);

	cairo_save(cr);
	if(rotate){
		cairo_translate(cr, core->curx, core->cury);
		cairo_rotate(cr, atan2(core->cury - core->lasty, core->curx - core->lastx));

		cx = core->width / 2.;
		cy = core->height / 2.;
		cairo_translate(cr, -cx, -cy);
	}

	gdk_cairo_set_source_pixbuf(cr, pixbuf,
								core->curx - core->width / 2.,
								core->cury - core->height / 2.);

	cairo_paint(cr);
	cairo_restore(cr);
}

static void draw_ellip(PaintCore *core, cairo_t *cr)
{
	BrushInfo *binfo = core->binfo;
	GdkRectangle r;

	r.x = core->curx - core->width / 2.;
	r.y = core->cury - core->height / 2.;
	r.width = core->width;
	r.height = core->width;

	cairo_save (cr);
	draw_ellipse(cr, r, &fg, binfo->filled ? &fg : NULL);
	cairo_restore (cr);

}

static void brush_gradient_draw(cairo_t *cr, double cx, double cy, PaintCore *core)
{
	double brush_width = core->width;
	double radius, radius2;
	cairo_pattern_t *rp;

	radius = brush_width / 2.;
	if(radius < 1.){
		radius = 1.;
		cx += .5;
		cy += .5;
	}

	radius2 = .125 * radius;

	rp = cairo_pattern_create_radial(cx, cy, radius, cx, cy, radius2);  
	cairo_pattern_add_color_stop_rgba(rp, 1,  fg.r, fg.g, fg.b, fg.a);
	cairo_pattern_add_color_stop_rgba(rp, 0,  fg.r, fg.g, fg.b, 0);
	cairo_set_source(cr, rp);
	cairo_arc(cr, cx, cy, radius, 0, M_PI * 2.);

	cairo_fill(cr);
	cairo_pattern_destroy(rp);   
}

static void draw_gradient(PaintCore *core, cairo_t *cr)
{
	double x, y;

	x = core->curx;
	y = core->cury;

	cairo_save (cr);
	brush_gradient_draw(cr, x, y, core);
	cairo_restore (cr);

}

static void draw_crosshatch(PaintCore *core, cairo_t *cr)
{
	double x, y, w, h, i;

	w = ABS(core->width);
	h = ABS(core->width);

	if(!h || !w) return;

	cairo_save (cr);

	cairo_set_line_width(cr, 1.);

	cairo_translate(cr, core->curx, core->cury);
	x = -core->width / 2.;
	y = -core->height / 2.;
	x += .5;
	y += .5;
	cairo_rotate(cr, M_PI / 180. * 45.);
	for(i = 6; i < (int)h; i += 6){
		cairo_move_to(cr, x, y + i);
		cairo_line_to(cr, x + w, y + i);
		cairo_stroke(cr);
	}
	for(i = 6; i < (int)w; i += 6){
		cairo_move_to(cr, x + i, y);
		cairo_line_to(cr, x + i, y + h);
		cairo_stroke(cr);
	}

	cairo_restore (cr);
}

static void brush_draw_crayon(PaintCore *core, cairo_t *cr)
{
	double x, y, rx, ry;
	double radius, dist;
	int i;

	x = core->curx - core->width / 2.;
	y = core->cury - core->height / 2.;
	radius = core->width / 2.;

	cairo_save (cr);
	cairo_set_line_width(cr, 1.);

	for(i = 0; i < (int)(core->width / 2.); i++){
		rx = (double)g_random_int_range(0, core->width);
		ry = (double)g_random_int_range(0, core->height);
		rx = x + rx;
		ry = y + ry;
		dist = sqrt((core->curx - rx) * (core->curx - rx) +
			   (core->cury - ry) * (core->cury - ry));
		if(dist <= radius){
			cairo_rectangle(cr, rx, ry, 1., 1.);
			cairo_fill(cr);
		}
	}
	cairo_restore (cr);

}

static void slash_brush(PaintCore *core, cairo_t *cr)
{
	double x, y, w, h;
	BrushInfo *binfo = core->binfo;

	x = core->curx - core->width / 2.;
	y = core->cury - core->height / 2.;
	w = core->width;
	h = core->width;

	cairo_save (cr);

	if(BRUSH_BSLASH == binfo->type){
		cairo_move_to(cr, x, y);
		cairo_line_to(cr, x + w, y + h);
	}
	else{
		cairo_move_to(cr, x, y + h);
		cairo_line_to(cr, x + w, y);
	}

	cairo_stroke(cr);
	cairo_restore (cr);
	
}

static void draw_rect(PaintCore *core, cairo_t *cr)
{
	BrushInfo *binfo = core->binfo;
	GdkRectangle r;

	r.x = core->curx - core->width / 2.;
	r.y = core->cury - core->height / 2.;
	r.width = core->width;
	r.height = core->width;

	cairo_save (cr);
	draw_rectangle(cr, r, &fg, binfo->filled ? &fg : NULL);
	cairo_restore (cr);
}

static void draw_star(PaintCore *core, cairo_t *cr)
{
	double x, y, size, a = 0., cx, cy;
	int i;
	GdkPoint p = {0};

	size = ABS(core->width);

	if(!(size > 0.)) return;

	cairo_save (cr);

	cairo_set_line_width(cr, 1.);

	cx = core->curx + .5;
	cy = core->cury + .5;
	x = cx;
	y = cy;
	y -= (size / 2);

	for (i = 0; i < 9; i++){
		cairo_move_to(cr, (double)cx, (double)cy);

		p = draw_calc_rotate(a, (double)(x - cx), (double)(y - cy));
		cairo_line_to(cr, (double)p.x + cx, (double) p.y + cy);
		cairo_stroke(cr);

		a += 45.;
	}

	cairo_restore (cr);
}

static void draw_pent(PaintCore *core, cairo_t *cr)
{
	BrushInfo *binfo = core->binfo;

	cairo_save (cr);
	draw_polygon(cr, (int)core->curx, (int)core->cury,
				(int)core->curx, (int)(core->cury - core->height / 2.),
				5, &fg, binfo->filled ? &fg : NULL, NULL);
	cairo_restore (cr);

}

 // Make sure spacing is not 0!
static void paint_core_interpolate (PaintCore *paint_core, cairo_t *cr)
{
	int n;
	double dx, dy;
	double left;
	double t;
	double initial;
	double dist;
	double total;

	dx = paint_core->curx - paint_core->lastx;
	dy = paint_core->cury - paint_core->lasty;

	if (!dx && !dy)
		return;

	dist = sqrt (SQR (dx) + SQR (dy));
	total = dist + paint_core->distance;
	initial = paint_core->distance;

	while (paint_core->distance < total) {
		n = (int) (paint_core->distance / paint_core->spacing + 1.0 + EPSILON);
		left = n * paint_core->spacing - paint_core->distance;
		paint_core->distance += left;

		if (paint_core->distance <= total) {
			t = (paint_core->distance - initial) / dist;

			paint_core->curx = paint_core->lastx + dx * t;
			paint_core->cury = paint_core->lasty + dy * t;
			(* paint_core->paint_func) (paint_core, cr);

			paint_core->x1 = MIN(paint_core->curx, paint_core->x1);
			paint_core->y1 = MIN(paint_core->cury, paint_core->y1);
			paint_core->x2 = MAX(paint_core->curx, paint_core->x2);
			paint_core->y2 = MAX(paint_core->cury, paint_core->y2);
		}
	}

	paint_core->distance = total;
	paint_core->curx = paint_core->lastx + dx;
	paint_core->cury = paint_core->lasty + dy;
}

static gboolean brush_button_press(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	if(1 == event->button){
		GdkRectangle copy = {0};
		cairo_t *cr;

		copy.width = cairo_image_surface_get_width(dibuja_get_surface());
		copy.height = cairo_image_surface_get_height(dibuja_get_surface());
		undo_tmp_surf = dibuja_copy_from_surface(&copy);

		pc.startx = event->x;
		pc.starty = event->y;
		pc.curx = event->x;
		pc.cury = event->y;
		pc.lastx = event->x;
		pc.lasty = event->y;
		pc.x2 = pc.x1 = pc.startx;
		pc.y2 = pc.y1 = pc.starty;

		dibuja_get_fg_sccolor(&fg);
		dibuja_get_bg_sccolor(&bg);

		cr = dibuja_get_cr(CR_SURFACE);
		cairo_set_line_width(cr, 1.);

		pc.paint_func(&pc, cr);

		cairo_destroy(cr);
	}

	return TRUE;
}

static gboolean brush_motion(GtkWidget *widget, GdkEventMotion *event, gpointer data)
{

	if(event->state & GDK_BUTTON1_MASK){
		cairo_t *cr;
		GdkRectangle area;

		pc.curx = event->x;
		pc.cury = event->y;

		brush_set_update_rect(&area);

		cr = dibuja_get_cr(CR_SURFACE);
		cairo_set_line_width(cr, 1.);

		paint_core_interpolate (&pc, cr);

		pc.lastx = event->x;
		pc.lasty = event->y;

		cairo_destroy(cr);

		gdk_window_invalidate_rect(gtk_widget_get_window(widget), &area, FALSE);
		
	}

	return TRUE;
}

static gboolean brush_button_release(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	if(1 == event->button && undo_tmp_surf){
		GdkRectangle area;
		cairo_surface_t *copy_from_undo_tmp_surf;

		rect_from_pts(&area, (int)pc.x1, (int)pc.y1, (int)pc.x2, (int)pc.y2);
		resize_rect(&area, (int)pc.width + 2, (int)pc.height + 2);//  + 2 for alias

		copy_from_undo_tmp_surf = draw_copy_area(undo_tmp_surf, area.x, area.y, area.width, area.height);
		undo_add_from_surface(SIMPLE_CANVAS(widget), copy_from_undo_tmp_surf,
								area, _("Brush"), UNDO_TYPE_BRUSH);

	}
	if(undo_tmp_surf){
		cairo_surface_destroy(undo_tmp_surf);
		undo_tmp_surf = NULL;
	}

	return TRUE;
}

static gboolean brush_expose(GtkWidget *widget, cairo_t *cr, gpointer data)
{
	return TRUE;
}

static void brush_set_type(BrushInfo *bi)
{
	if(!bi)
		return;

	brush_finish();
	switch(bi->type){
		case BRUSH_RECTANGLE:
			brush_type = bi->type;
			util_spin_set_val("spinbutton_brush_spacing", bi->spacing);
			util_spin_set_val("spinbutton_brush_width", bi->size);
			pc.paint_func = draw_rect;
			break;
		case BRUSH_ELLIPSE:
			brush_type = bi->type;
			util_spin_set_val("spinbutton_brush_spacing", bi->spacing);
			util_spin_set_val("spinbutton_brush_width", bi->size);
			pc.paint_func = draw_ellip;
			break;
		case BRUSH_GRADIENT:
			brush_type = bi->type;
			util_spin_set_val("spinbutton_brush_spacing", bi->spacing);
			util_spin_set_val("spinbutton_brush_width", bi->size);
			pc.paint_func = draw_gradient;
			break;
		case BRUSH_CROSSHATCH:
			brush_type = bi->type;
			util_spin_set_val("spinbutton_brush_spacing", bi->spacing);
			util_spin_set_val("spinbutton_brush_width", bi->size);
			pc.paint_func = draw_crosshatch;
			break;
		case BRUSH_RANDDOTS:
			brush_type = bi->type;
			util_spin_set_val("spinbutton_brush_spacing", bi->spacing);
			util_spin_set_val("spinbutton_brush_width", bi->size);
			pc.paint_func = brush_draw_crayon;
			break;
		case BRUSH_BSLASH:
		case BRUSH_FSLASH:
			brush_type = bi->type;
			util_spin_set_val("spinbutton_brush_spacing", bi->spacing);
			util_spin_set_val("spinbutton_brush_width", bi->size);
			pc.paint_func = slash_brush;
			break;
		case  BRUSH_PENT:
			brush_type = bi->type;
			util_spin_set_val("spinbutton_brush_spacing", bi->spacing);
			util_spin_set_val("spinbutton_brush_width", bi->size);
			pc.paint_func = draw_pent;
			break;
		case BRUSH_STAR:
			brush_type = bi->type;
			util_spin_set_val("spinbutton_brush_spacing", bi->spacing);
			util_spin_set_val("spinbutton_brush_width", bi->size);
			pc.paint_func = draw_star;
			break;
#if 0
		case BRUSH_IMAGE:
			brush_type = bi->type;
			util_spin_set_val("spinbutton_brush_spacing", bi->spacing);
			util_spin_set_val("spinbutton_brush_width", bi->size);
			pc.paint_func = brush_draw_image;
			brush_image_darea_show(TRUE);
			break;
#endif
		default:
			break;
	}
	brush_type = bi->type;
	pc.binfo = bi;
}

/* Set brush to currently selected item */
static int brush_set_current_brush(GtkImage *img)
{
	int ret = 0;

	if(GTK_IS_IMAGE(img)){
		BrushInfo *bi = (BrushInfo *)g_object_get_data(G_OBJECT(img), "BrushInfo");

		if(bi){
			if(BRUSH_IMAGE != bi->type){
				brush_set_type(bi);
				gtk_widget_set_sensitive(
						get_widget("spinbutton_brush_width"), TRUE);
				brush_image_darea_show(FALSE);
			}
			else{
				GtkWidget *darea = get_widget("img_brush_darea");
				GdkPixbuf *pixbuf;

				pixbuf = darea_get_img(darea);
				brush_brush_img_setup(pixbuf);

				/* Disable brush width spinbutton because images
				   are not resizable */
				gtk_widget_set_sensitive(
								get_widget("spinbutton_brush_width"), TRUE);
				brush_type = BRUSH_IMAGE;
				pc.paint_func = brush_draw_image;
				pc.binfo->imgwidget = GTK_WIDGET(img);
				ret = 1;
				gtk_widget_set_sensitive(
						get_widget("spinbutton_brush_width"), FALSE);
				brush_image_darea_show(TRUE);
			}
		}
	}

	return ret;
}

static void brush_restore_image_brush(GtkWidget *menu, GtkWidget *darea)
{
	GdkPixbuf *pixbuf;

	g_return_if_fail(GTK_IS_DRAWING_AREA(darea));

	pixbuf = GDK_PIXBUF(g_object_steal_data(G_OBJECT(darea), "pixbuf"));
	if(GDK_IS_PIXBUF(pixbuf)){
		g_object_unref(pixbuf);
		pixbuf = NULL;
	}

	pixbuf = GDK_PIXBUF(g_object_get_data(G_OBJECT(darea), "default-image"));
	brush_brush_img_setup(pixbuf);
	brush_image_darea_show(TRUE);
	gtk_widget_queue_draw(darea);
}

static GtkWidget *create_menu(GtkWidget *darea)
{
	GtkWidget *menu;
	GtkWidget *restore;

	menu = gtk_menu_new();

	restore = gtk_menu_item_new_with_label(_("Restore Image Brush"));
	gtk_widget_show(restore);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), restore);

	g_signal_connect(G_OBJECT(restore), "activate",
					 G_CALLBACK(brush_restore_image_brush), (gpointer)darea);

	return menu;
}

static gboolean image_brush_pressed_cb(GtkWidget *widget, GdkEventButton *event,
									   GtkWidget *menu)
{
	if(3 == event->button){
#if GTK_CHECK_VERSION(3, 22, 0)
		gtk_menu_popup_at_pointer(GTK_MENU(menu), (GdkEvent *)event);
#else
		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
					   event->button, event->time);
#endif
		return TRUE;
	}

	return FALSE;
}

static gboolean image_brush_darea_draw(GtkWidget *darea, cairo_t *cr)
{
	GdkPixbuf *pixbuf = NULL;

	pixbuf = darea_get_img(darea);
	if(!GDK_IS_PIXBUF(pixbuf)){
		return TRUE;
	}

	gdk_cairo_set_source_pixbuf(cr, pixbuf, 0., 0.);
	cairo_paint(cr);

	return TRUE;
}

#if !GTK_CHECK_VERSION(3, 0, 0)
static gboolean image_brush_darea_expose(GtkWidget *darea, GdkEventExpose *event)
{
	cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(darea));

	image_brush_darea_draw(darea, cr);
	cairo_destroy(cr);

	return TRUE;
}
#endif

static void brush_init_image_brush(GtkWidget *button)
{
	GtkWidget *menu;
	GdkPixbuf *default_img = NULL;
	char *filename = NULL;
	GtkWidget *darea = get_widget("img_brush_darea");

	filename = g_build_filename(DEF_BRUSH_DIR, "smiley.png", NULL);
	default_img = gdk_pixbuf_new_from_file((const char *)filename, NULL);
	g_object_set_data_full(G_OBJECT(darea), "default-image",
						   (gpointer)default_img, g_object_unref);
	g_free(filename);
	brush_image_darea_show(FALSE);

#if GTK_CHECK_VERSION(3, 0, 0)
	g_signal_connect(G_OBJECT(darea), "draw",
					 G_CALLBACK(image_brush_darea_draw),
					 NULL);
#else
	g_signal_connect(G_OBJECT(darea), "expose-event",
					 G_CALLBACK(image_brush_darea_expose),
					 NULL);
#endif
	menu = create_menu(darea);
	g_signal_connect(G_OBJECT(darea), "button-press-event",
					 G_CALLBACK(image_brush_pressed_cb),
					 (gpointer)menu);
}

static void brush_add_brush(GtkImage *img, BrushInfo bi, char *filename)
{
	GdkPixbuf *pixbuf = NULL;
	BrushInfo *pbi = NULL;

	if(!filename){
		g_message("Null filename for image: %p\n", img);
		return;
	}
	if(!GTK_IS_IMAGE(img)){
		g_message("Invalid GtkImage: %s\n", filename);
		return;
	}

	pixbuf = file_load_pixbuf(filename, FALSE);
	if(!GDK_IS_PIXBUF(pixbuf)) {
		g_message("could not get image for brush: %s\n", filename);
		g_message("from file %s\n", filename);
		return; 
	}

	gtk_widget_show(GTK_WIDGET(img));
	pbi = g_malloc(sizeof(BrushInfo));
	pbi->type = bi.type;
	pbi->spacing = bi.spacing;
	pbi->filled = bi.filled;
	pbi->size = bi.size;
	g_strlcpy(pbi->desc, bi.desc, sizeof(bi.desc));
	pbi->imgwidget = GTK_WIDGET(img);

	g_object_set_data_full(G_OBJECT(img), "BrushInfo", (gpointer)pbi, (GDestroyNotify)g_free);
	gtk_image_set_from_pixbuf(img, pixbuf);

	if(BRUSH_IMAGE == bi.type){
		brush_init_image_brush(gtk_widget_get_parent(GTK_WIDGET(img)));
	}

	g_object_unref(pixbuf);
}

void brush_init(GtkBuilder *builder)
{
	int i;
	BrushInfo stock_brushes[] = {
		{ BRUSH_ELLIPSE,      1,  1,  18, "13-ellipse.png"       , NULL },
		{ BRUSH_RECTANGLE,    1,  1,  18, "12-rectangle.png"     , NULL },
		{ BRUSH_ELLIPSE,     18,  0,  18, "00-ellipse.png"       , NULL },
		{ BRUSH_RECTANGLE,   18,  0,  18, "01-rectangle.png"     , NULL },
		{ BRUSH_BSLASH,       1,  0,  22, "02-back-slash.svg"    , NULL },
		{ BRUSH_FSLASH,       1,  0,  22, "03-fwd-slash.svg"     , NULL },
		{ BRUSH_PENT,        18,  0,  18, "04-pentagon.svg"      , NULL },
		{ BRUSH_STAR,        18,  0,  18, "05-star.svg"          , NULL },
		{ BRUSH_GRADIENT,     6,  0,  22, "06-shpere.svg"        , NULL },
		{ BRUSH_CROSSHATCH,  22,  0,  22, "11-crosshatch.png"    , NULL },
		{ BRUSH_RANDDOTS,     0,  0,  20, "rand_dots.png"        , NULL },
		{ BRUSH_IMAGE,       22,  0,  22, "image-brush.png"      , NULL },
		{ BRUSH_LAST,         0,  0,   0, "\0"                   , NULL }
	};
	GtkWidget *darea = GTK_WIDGET(gtk_builder_get_object(builder, "img_brush_display"));

	// Save the widget
	g_object_set_data(G_OBJECT(get_window()), "img_brush_darea", (gpointer)darea);

	for (i = 0; stock_brushes[i].desc[0] != '\0'; i++){
		gchar *filename;
		gchar *imgname = NULL;
		gboolean is_dir;
		GtkImage *img = NULL;

		filename = g_build_filename (DEF_BRUSH_DIR, stock_brushes[i].desc, NULL);

		if(filename){
			filename = g_strchomp(filename);
			is_dir = g_file_test (filename, G_FILE_TEST_IS_DIR);
			if(!is_dir){

				imgname = g_strdup_printf("img_brush%d", i);
				img = GTK_IMAGE (gtk_builder_get_object (builder, imgname));
				brush_add_brush(img, stock_brushes[i], filename);

				img = NULL;

				g_free(imgname);
			}
			g_free (filename);
		}
	}

	brush_set_current_brush(GTK_IMAGE (gtk_builder_get_object (builder, "img_brush0")));

	// Save the widget
	g_object_set_data(G_OBJECT(get_window()), "img-brush", 
					 (gpointer)gtk_builder_get_object (builder, "rb_brush11"));
}

void rb_brush_toggled(GtkToggleButton *button)
{
	if(gtk_toggle_button_get_active(button)){
		GtkImage *img;

		img = GTK_IMAGE(gtk_bin_get_child(GTK_BIN(button)));
		brush_set_current_brush(img);
	}
}
