/* 
 * (c) 2014 - 2023 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>
#include "color.h"
#include "dibuja.h"
#include "util.h"
#include "dibuja-colorsel.h"
#include "palette-color.h"
#include "dibujacolorbutton.h"

#ifndef MOUSE_RIGHT_BUTTON
	#define  MOUSE_RIGHT_BUTTON	(3)
#endif
#ifndef MOUSE_LEFT_BUTTON
	#define  MOUSE_LEFT_BUTTON	(1)
#endif

static void tool_color_button_press_event(PaletteColor *pc, gint button);
gboolean colorbutton_right_click(GtkWidget *colorbutton, GdkEventButton *event,
								 gpointer data);

void color_sccolor2rgba(SCColor *color,
						guchar *r, guchar *g, guchar *b, guchar *a)
{
	if(color){
		if(r)
			*r = (guchar)(color->r * 255.);
		if(g)
			*g = (guchar)(color->g * 255.);
		if(b)
			*b = (guchar)(color->b * 255.);
		if(a)
			*a = (guchar)(color->a * 255.);
	}
}

void color_button_get_sccolor(GtkWidget *button, SCColor *color)
{
	g_return_if_fail(DIBUJA_IS_COLOR_BUTTON(button));
	g_return_if_fail(color != NULL);

	dibuja_color_button_get_color(DIBUJA_COLOR_BUTTON(button), color);
}

static void color_get_color_button_sccolor(char *btn_name, SCColor *color)
{
	GtkWidget *button = get_widget(btn_name);

	color_button_get_sccolor(button, color);
}

static void color_button_color_set(char *btn_name, SCColor *color)
{
	GtkWidget *btn = get_widget(btn_name);

	if(!btn) return;

	dibuja_color_button_set_color(DIBUJA_COLOR_BUTTON(btn), color);
}

// Notify fg SCColor changed
static void button_fg_notify_color(DibujaColorButton *btn)
{
	SCColor color;

	color_get_color_button_sccolor("colorbutton_fg", &color);
	dibuja_set_fg_sccolor(&color);
}

// Notify bg SCColor changed
static void button_bg_notify_color(DibujaColorButton *btn)
{
	SCColor color;

	color_get_color_button_sccolor("colorbutton_bg", &color);
	dibuja_set_bg_sccolor(&color);
}

static GtkWidget *create_palette_color(SCColor *color, char *name)
{
	GtkWidget *pal_color = palette_color_new(color);

	g_object_set_data(G_OBJECT(dibuja_get_canvas()), name, (gpointer)pal_color);
	g_signal_connect(G_OBJECT(pal_color), "color-selected",
							 G_CALLBACK(tool_color_button_press_event), NULL);

	return pal_color;
}

static
GtkWidget *color_create_colorbutton(const char *name, GCallback callback)
{
	GtkWidget *button;

	button = dibuja_color_button_new(NULL);
	gtk_widget_set_size_request(button, -1, 25);
	// Save the button
	g_object_set_data(G_OBJECT(get_window()), name, (gpointer)button);
	g_signal_connect(G_OBJECT(button), "button-color-selected", callback, NULL);
	g_signal_connect(G_OBJECT(button), "button-press-event",
					 G_CALLBACK(colorbutton_right_click), NULL);
	gtk_widget_show_all(button);

	return button;
}

void color_init(GtkBuilder *builder, double fg[4], double bg[4])
{
	GtkWidget *color_button_fg;
	GtkWidget *color_button_bg;
	GtkWidget *button, *table, *child;
	char name[40];
	int i = 0;
	int left, top;
	SCColor fgcolor = {fg[0], fg[1], fg[2], fg[3]};
	SCColor bgcolor = {bg[0], bg[1], bg[2], bg[3]};
	char *tooltip = N_("Left click to set this color\n"
					"as the foreground color.\n\n"
					"Right click to set this color\n"
					"as the background color.");
	char *tt_fore = N_("Foreground Color.\n\nRight click to swap\n"
					   "background and\nforeground color");
	char *tt_back = N_("Background Color.\n\nRight click to swap\n"
					   "background and\nforeground color");
	table = GTK_WIDGET (gtk_builder_get_object (builder, "table_col_pal"));

	color_button_fg = color_create_colorbutton("colorbutton_fg",
								G_CALLBACK(button_fg_notify_color));
	dib_table_attach(table, color_button_fg, 0, 2, 0, 1, DIB_FILL, DIB_FILL, 0, 0);
	gtk_widget_set_tooltip_text(color_button_fg, _(tt_fore));

	color_button_bg = color_create_colorbutton("colorbutton_bg",
								G_CALLBACK(button_bg_notify_color));
	dib_table_attach(table, color_button_bg, 0, 2, 1, 2, DIB_FILL, DIB_FILL, 0, 0);
	gtk_widget_set_tooltip_text(color_button_bg, _(tt_back));

#if !GTK_CHECK_VERSION(3, 0, 0)
	gtk_table_resize(GTK_TABLE(table), 12, 2);
#endif
	for(top = 2; top < 12; top++){
		for(left = 0; left < 2; left++){
			sprintf(name, "button%d", i);
			button = create_palette_color((SCColor *)&(default_colors[i]), name);
			child = palette_color_get_child(PALETTE_COLOR(button));
			gtk_widget_set_tooltip_text(child, _(tooltip));

			dib_table_attach(table, button,
							  left, left + 1,
							  top, top + 1,
							  DIB_FILL, DIB_FILL, 0, 0);
			i++;
		}
	}

	color_button_color_set("colorbutton_fg", &fgcolor);
	color_button_color_set("colorbutton_bg", &bgcolor);
}

// Palette color clicked
static void tool_color_button_press_event(PaletteColor *pc, gint button)
{
	SCColor color = {0};

	palette_color_get_color(pc, &color);

	if(MOUSE_LEFT_BUTTON == button){
		color_button_color_set("colorbutton_fg", &color);
	}
	else if(MOUSE_RIGHT_BUTTON == button){
		color_button_color_set("colorbutton_bg", &color);
	}
}

// Switch the colors of the color buttons
gboolean colorbutton_right_click(GtkWidget *colorbutton, GdkEventButton *event,
								 gpointer data)
{
	if(MOUSE_RIGHT_BUTTON == event->button){
		SCColor fg, bg;

		color_get_color_button_sccolor("colorbutton_fg", &fg);
		color_get_color_button_sccolor("colorbutton_bg", &bg);

		color_button_color_set("colorbutton_fg", &bg);
		color_button_color_set("colorbutton_bg", &fg);
	}
	return FALSE;
}

/* TODO */
/*static int color_edit_dialog_set_palette(gchar *palette)
{
	if(!palette) return 0;

	return 1;
}*/

/* Has default color palette on startup */
void color_edit_dialog(GtkWidget *widget)
{
	gint result;
	static GtkWidget *dialog = NULL;

	if(!GTK_IS_DIALOG(dialog)){
		dialog = dibuja_color_dialog_new(get_window());
	}

	result = gtk_dialog_run(GTK_DIALOG(dialog));

	if(GTK_RESPONSE_ACCEPT == result){
		SCColor *colors = NULL;
		int i, ncolors;
		GtkWidget *button;
		char name[40];
		GtkWidget *canvas = dibuja_get_canvas();

		colors = dibuja_colorsel_get_colors(&ncolors);

		if(colors){
			for(i = 0; i < ncolors; i++){
				sprintf(name, "button%d", i);
				button = GTK_WIDGET (g_object_get_data(G_OBJECT(canvas), name));
				if(PALETTE_IS_COLOR(button)){
					palette_color_set_color(PALETTE_COLOR(button), &colors[i]);
				}
			}
			g_free(colors);
		}
	}
	gtk_widget_hide(dialog);
}



