/*
 * dibujacolorbutton.c
 * 
 * Copyright 2023 Juan Balderas
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>
#include "dibujacolorbutton.h"
#include "palette-color.h"
#include "util.h"


#if GLIB_CHECK_VERSION(2, 58, 0)
  #define dibuja_color_button_get_instance_priv  dibuja_color_button_get_instance_private
#else
  #define DIBUJA_COLOR_BUTTON_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
                                          DIBUJA_TYPE_COLOR_BUTTON,                 \
                                          DibujaColorButtonPrivate))

  #define dibuja_color_button_get_instance_priv	DIBUJA_COLOR_BUTTON_GET_PRIVATE
#endif

struct _DibujaColorButtonPrivate
{
  GtkWidget *dialog;
  GtkWidget *color_sel;
  GtkWidget *pal_col;
  gboolean dlg_is_init;
};

enum {
	BUTTON_COLOR_SELECTED,
	LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = { 0 };

static void dibuja_color_button_finalize (GObject *object);


G_DEFINE_TYPE_WITH_CODE (DibujaColorButton, dibuja_color_button, GTK_TYPE_BUTTON, G_ADD_PRIVATE(DibujaColorButton))

static void dbj_color_button_clicked(DibujaColorButton *button);

static void
dibuja_color_button_class_init (DibujaColorButtonClass *klass)
{
  GObjectClass *g_object_class;
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

  g_object_class = G_OBJECT_CLASS (klass);

  g_object_class->finalize = dibuja_color_button_finalize;

    signals[BUTTON_COLOR_SELECTED] =
    g_signal_new ("button-color-selected",
		  G_OBJECT_CLASS_TYPE (widget_class),
		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
		  G_STRUCT_OFFSET (DibujaColorButtonClass, button_color_selected),
		  NULL, NULL,
		  g_cclosure_marshal_VOID__VOID,
		  G_TYPE_NONE, 0);
}


static void
dibuja_color_button_finalize (GObject *object)
{
  DibujaColorButton *button;

  g_return_if_fail (DIBUJA_IS_COLOR_BUTTON (object));

  button = DIBUJA_COLOR_BUTTON (object);
  if(button->priv->dialog != NULL){
    gtk_widget_destroy(button->priv->dialog);
  }

  G_OBJECT_CLASS (dibuja_color_button_parent_class)->finalize (object);
}

static void destroy_dlg(GtkWidget *widget, gpointer data)
{
	DibujaColorButton *button = DIBUJA_COLOR_BUTTON(data);

	button->priv->dialog = NULL;
}

static void
dibuja_color_button_init (DibujaColorButton *button)
{
  DibujaColorButtonPrivate *priv;
  SCColor color = {0, 0, 0, 1.};

  priv = dibuja_color_button_get_instance_priv(button);
  button->priv = priv; 

  g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(dbj_color_button_clicked), NULL);

  priv->pal_col = palette_color_new(&color);
#if GTK_CHECK_VERSION(3, 0, 0)
  gtk_widget_set_size_request(GTK_WIDGET(priv->pal_col), -1, 15);
  gtk_widget_set_size_request(GTK_WIDGET(button), -1, 20);
#else
  gtk_widget_set_size_request(GTK_WIDGET(priv->pal_col), -1, 20);
#endif
  gtk_container_add(GTK_CONTAINER(button), priv->pal_col);

  priv->dlg_is_init = FALSE;

G_GNUC_BEGIN_IGNORE_DEPRECATIONS
  priv->dialog = gtk_color_selection_dialog_new(_("Pick a Color"));
  priv->color_sel = gtk_color_selection_dialog_get_color_selection(GTK_COLOR_SELECTION_DIALOG(priv->dialog));
  gtk_color_selection_set_has_opacity_control(GTK_COLOR_SELECTION(priv->color_sel), TRUE);
  g_signal_connect(G_OBJECT(priv->dialog), "destroy", G_CALLBACK(destroy_dlg), (gpointer)button);
G_GNUC_END_IGNORE_DEPRECATIONS

}

GtkWidget *
dibuja_color_button_new (SCColor *color)
{
  DibujaColorButton *button;

  button = DIBUJA_COLOR_BUTTON(g_object_new (DIBUJA_TYPE_COLOR_BUTTON, NULL));
  if(color){
  	dibuja_color_button_set_color(button, color);
  }
  else{
  	SCColor c = {0., 0., 0., 1.};
  	dibuja_color_button_set_color(button, &c);
  }

  return GTK_WIDGET(button);
}

static void ensure_dialog(DibujaColorButton *button)
{
	GtkWindow *parent;
	DibujaColorButtonPrivate *priv = button->priv;

G_GNUC_BEGIN_IGNORE_DEPRECATIONS
	g_return_if_fail(GTK_IS_COLOR_SELECTION_DIALOG(priv->dialog));
G_GNUC_END_IGNORE_DEPRECATIONS

	parent = GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(button)));
	gtk_window_set_transient_for(GTK_WINDOW(priv->dialog), parent);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(priv->dialog), TRUE);
	priv->dlg_is_init = TRUE;
#if GTK_CHECK_VERSION(3, 12, 0)
	util_set_select_dlg_button_image(GTK_DIALOG(priv->dialog), -5, STOCK_APPLY);
	util_set_select_dlg_button_image(GTK_DIALOG(priv->dialog), -6, STOCK_CANCEL);
#endif
}

static void color_update(DibujaColorButton *button, SCColor *color)
{
	palette_color_set_color(PALETTE_COLOR(button->priv->pal_col), color);
	g_signal_emit(button, signals[BUTTON_COLOR_SELECTED], 0);
}

static void dbj_color_button_clicked(DibujaColorButton *button)
{
	DibujaColorButtonPrivate *priv;
	int ret;

	g_return_if_fail(DIBUJA_IS_COLOR_BUTTON(button));

	priv = button->priv;
	if(!(priv->dlg_is_init))
		ensure_dialog (button);

	ret = gtk_dialog_run(GTK_DIALOG(priv->dialog));
	if(GTK_RESPONSE_OK == ret){
		SCColor color;

		dibuja_color_button_get_color(button, &color);
		color_update(button, &color);
	}
	gtk_widget_hide(priv->dialog);
}

void dibuja_color_button_set_color(DibujaColorButton *button, SCColor *color)
{
	DibujaColorButtonPrivate *priv;

	g_return_if_fail(DIBUJA_IS_COLOR_BUTTON(button));
	g_return_if_fail(color != NULL);

	priv = button->priv;
	palette_color_set_color(PALETTE_COLOR(priv->pal_col), color);
#if GTK_CHECK_VERSION(3, 0, 0)
G_GNUC_BEGIN_IGNORE_DEPRECATIONS
	gtk_color_selection_set_current_rgba(GTK_COLOR_SELECTION(priv->color_sel), (GdkRGBA *)color);
G_GNUC_END_IGNORE_DEPRECATIONS
#else
	GdkColor gcol;
	guint16 alpha;

	gcol.red   = (guint16)(color->r  *  65535.);
	gcol.green = (guint16)(color->g  *  65535.);
	gcol.blue  = (guint16)(color->b  *  65535.);
	alpha      = (guint16)(color->a  *  65535.);

	gtk_color_selection_set_current_alpha(GTK_COLOR_SELECTION(priv->color_sel), alpha);
	gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(priv->color_sel), &gcol);
#endif

	color_update(button, color);
}

void dibuja_color_button_get_color(DibujaColorButton *button, SCColor *color)
{
	DibujaColorButtonPrivate *priv;

	g_return_if_fail(DIBUJA_IS_COLOR_BUTTON(button));
	g_return_if_fail(color != NULL);

	priv = button->priv;
#if GTK_CHECK_VERSION(3, 0, 0)
G_GNUC_BEGIN_IGNORE_DEPRECATIONS
	gtk_color_selection_get_current_rgba(GTK_COLOR_SELECTION(priv->color_sel), (GdkRGBA *)color);
G_GNUC_END_IGNORE_DEPRECATIONS
#else
	GdkColor gcol;
	guint16 alpha;

	alpha = gtk_color_selection_get_current_alpha(GTK_COLOR_SELECTION(priv->color_sel));
	gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(priv->color_sel), &gcol);
	color->r = (gdouble)gcol.red    /  65535.;
	color->g = (gdouble)gcol.green  /  65535.;
	color->b = (gdouble)gcol.blue   /  65535.;
	color->a = (gdouble)alpha       /  65535.;
#endif
}
