/*
 * dibujafontbutton.c
 * 
 * Copyright 2023 Juan Balderas
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>
#include "dibujafontbutton.h"
#include "util.h"


#if GLIB_CHECK_VERSION(2, 58, 0)
	#define dbj_font_button_get_instance_priv  dbj_font_button_get_instance_private
#else
	#define DBJ_FONT_BUTTON_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
	                                        DBJ_TYPE_FONT_BUTTON,                 \
	                                        DbjFontButtonPrivate))

	#define dbj_font_button_get_instance_priv	DBJ_FONT_BUTTON_GET_PRIVATE
#endif

struct _DbjFontButtonPrivate{
	GtkWidget *dialog;
	char *fontname;
	gboolean dlg_is_init;
};

enum {
	FONT_SELECTED,
	LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = { 0 };

static void dbj_font_button_run_dlg(DbjFontButton *button);
static void dbj_font_button_finalize (GObject *object);

G_DEFINE_TYPE_WITH_CODE (DbjFontButton, dbj_font_button, GTK_TYPE_BUTTON, G_ADD_PRIVATE(DbjFontButton))

static void dbj_font_button_class_init (DbjFontButtonClass *klass)
{
	GObjectClass *g_object_class;
	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

	g_object_class = G_OBJECT_CLASS (klass);

	g_object_class->finalize = dbj_font_button_finalize;

//	g_type_class_add_private ((gpointer)klass, sizeof (DbjFontButtonPrivate));

	signals[FONT_SELECTED] =
    g_signal_new ("font-selected",
		  G_OBJECT_CLASS_TYPE (widget_class),
		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
		  G_STRUCT_OFFSET (DbjFontButtonClass, font_selected),
		  NULL, NULL,
		  g_cclosure_marshal_VOID__VOID,
		  G_TYPE_NONE, 0);
}

static void dbj_font_button_finalize (GObject *object)
{
	DbjFontButton *button;

	g_return_if_fail (DBJ_IS_FONT_BUTTON (object));

	button = DBJ_FONT_BUTTON (object);
	if(button->priv->dialog != NULL){
	  gtk_widget_destroy(button->priv->dialog);
	}
	if(button->priv->fontname != NULL){
		g_free(button->priv->fontname);
	}

	G_OBJECT_CLASS (dbj_font_button_parent_class)->finalize (object);
}

static void destroy_dlg(GtkWidget *widget, gpointer data)
{
	DbjFontButton *button = DBJ_FONT_BUTTON(data);

	button->priv->dialog = NULL;
}

static void dbj_font_button_init (DbjFontButton *button)
{
	DbjFontButtonPrivate *priv = NULL;

	priv = button->priv = dbj_font_button_get_instance_priv(button);
	priv->dlg_is_init = FALSE;

#if GTK_CHECK_VERSION(3, 2, 0)
	G_GNUC_BEGIN_IGNORE_DEPRECATIONS
#endif
	priv->dialog = gtk_font_selection_dialog_new(_("Pick a Font"));
	gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(priv->dialog),
											"Sans 12");
	gtk_button_set_label(GTK_BUTTON(button), "Sans 12");
	g_signal_connect(G_OBJECT(priv->dialog), "destroy", G_CALLBACK(destroy_dlg), (gpointer)button);
#if GTK_CHECK_VERSION(3, 2, 0)
	G_GNUC_END_IGNORE_DEPRECATIONS
#endif
	g_signal_connect(G_OBJECT(button), "clicked",
					   G_CALLBACK(dbj_font_button_run_dlg), NULL);
}

GtkWidget *dbj_font_button_new (void)
{
	return GTK_WIDGET(g_object_new (DBJ_TYPE_FONT_BUTTON, NULL));
}

static void update_fontname(DbjFontButton *button, char *fontname)
{
	DbjFontButtonPrivate *priv = NULL;

	g_return_if_fail(fontname != NULL);

	priv = button->priv;
	if(priv->fontname)
		g_free(priv->fontname);

	priv->fontname = g_strdup(fontname);
	gtk_button_set_label(GTK_BUTTON(button), (const gchar *)fontname);
	g_signal_emit(button, signals[FONT_SELECTED], 0);
}

static void ensure_dialog(DbjFontButton *button)
{
	GtkWindow *parent;
	DbjFontButtonPrivate *priv = button->priv;

G_GNUC_BEGIN_IGNORE_DEPRECATIONS
	g_return_if_fail(GTK_IS_FONT_SELECTION_DIALOG(priv->dialog));
G_GNUC_END_IGNORE_DEPRECATIONS

	parent = GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(button)));
	gtk_window_set_transient_for(GTK_WINDOW(priv->dialog), parent);
	gtk_window_set_destroy_with_parent(GTK_WINDOW(priv->dialog), TRUE);
	priv->dlg_is_init = TRUE;

#if GTK_CHECK_VERSION(3, 12, 0)
	util_set_select_dlg_button_image(GTK_DIALOG(priv->dialog), -5, STOCK_APPLY);
	util_set_select_dlg_button_image(GTK_DIALOG(priv->dialog), -6, STOCK_CANCEL);
#endif
}

static void dbj_font_button_run_dlg(DbjFontButton *button)
{
	DbjFontButtonPrivate *priv;
	int ret;
	GtkDialog *dlg;

	g_return_if_fail(DBJ_IS_FONT_BUTTON(button));

	priv = button->priv;
	if(!(priv->dlg_is_init))
		ensure_dialog (button);

	ret = gtk_dialog_run(GTK_DIALOG(priv->dialog));
	if(GTK_RESPONSE_OK == ret){
		char *fontname = NULL;
#if GTK_CHECK_VERSION(3, 2, 0)
		G_GNUC_BEGIN_IGNORE_DEPRECATIONS
#endif
		dlg = GTK_DIALOG(priv->dialog);
		fontname = gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(dlg));
#if GTK_CHECK_VERSION(3, 2, 0)
		G_GNUC_END_IGNORE_DEPRECATIONS
#endif
		update_fontname(button, fontname);
		g_free(fontname);
	}
	gtk_widget_hide(priv->dialog);	
}

void dbj_font_button_set_font_name(DbjFontButton *button, char *fname)
{
	g_return_if_fail(DBJ_IS_FONT_BUTTON(button));
	g_return_if_fail(fname != NULL);
	g_return_if_fail('\0' != fname[0]);

#if GTK_CHECK_VERSION(3, 2, 0)
	G_GNUC_BEGIN_IGNORE_DEPRECATIONS
#endif
	GtkFontSelectionDialog *dlg;
	dlg = GTK_FONT_SELECTION_DIALOG(button->priv->dialog);
	gtk_font_selection_dialog_set_font_name(dlg, (const gchar *)fname);
#if GTK_CHECK_VERSION(3, 2, 0)
	G_GNUC_END_IGNORE_DEPRECATIONS
#endif

	update_fontname(button, fname);
}

char *dbj_font_button_get_font_name(DbjFontButton *button)
{
	g_return_val_if_fail(DBJ_IS_FONT_BUTTON(button), NULL);

	return button->priv->fontname;
}







