/* 
 * (c) 2014 - 2023 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>

#include "util.h"

#ifdef HAVE_GEXIV2
  #include "metadata.h"
#endif

#include "dibuja.h"
#include "help.h"
#include "effects.h"

static char comments[] =
	N_("Simple to use paint program like classic MS Paint or Paintbrush"
       "for Mac but for Linux");
static gchar *authors[2] = {"Juan Balderas", NULL};

void help_show_htm(GtkWidget *menu, gpointer data)
{
	GError *error = NULL;
	gboolean ret = FALSE;

#if GTK_CHECK_VERSION(3, 22, 0)
	ret = gtk_show_uri_on_window(get_window(), DIBUJA_HELP_URI,
								 GDK_CURRENT_TIME, &error);
#else
	ret = gtk_show_uri(gtk_widget_get_screen(GTK_WIDGET(get_window())),
					   DIBUJA_HELP_URI,
					   gtk_get_current_event_time(),
					   &error);
#endif
// debug
	g_message("[%s: %d] gtk_show_uri() returned %d, looking for help file \"%s\"\n",
			  __FILE__, __LINE__, ret, DIBUJA_HELP_URI);
	if(NULL != error){
		message_box_error(_("Error opening the help file. %s") , error->message);
		g_error_free(error);
	}
	else if(!ret){
		message_box_error(_("Error opening the help file \"%s\".") , DIBUJA_HELP_URI);
	}
}

void help_about_dialog_activate(GtkWidget *menu, gpointer data)
{
	GdkPixbuf *pb;
	gchar license[] = 
	"Dibuja\n"
	"\n"
	"This program is free software: you can redistribute it and/or modify\n"
	"it under the terms of the GNU General Public License as published by\n"
	"the Free Software Foundation, either version 3 of the License, or\n"
	"(at your option) any later version.\n"
	"\n"
	"This program is distributed in the hope that it will be useful,\n"
	"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
	"GNU General Public License for more details.\n"
	"\n"
	"You should have received a copy of the GNU General Public License\n"
	"along with this program.  If not, see <https://www.gnu.org/licenses/>.";

	char copy_right[] = "(c) " COPY_YEAR " Juan Balderas ";

	pb = gdk_pixbuf_new_from_file(DIBUJA_LOGO, NULL);
	gtk_show_about_dialog (GTK_WINDOW(gtk_widget_get_toplevel(menu)),
				"program-name"   ,  "Dibuja" ,
				"copyright"      ,  copy_right,
				"logo"           ,  pb,
				"version"        ,  PACKAGE_VERSION,
				"license"        ,  license,
				"authors"        ,  authors ,
				"title"          ,  _("About Dibuja") ,
				"comments"       , _(comments),
				/* Translator name for credit */
				"translator-credits", _("translator-credits"),
				NULL);
	if(GDK_PIXBUF(pb))
		g_object_unref(pb);
}

/* From gimp app/version.c */
static void
gimp_show_library_version(const gchar *package,
						  gint build_time_major,
						  gint build_time_minor,
						  gint build_time_micro,
						  gint run_time_major,
						  gint run_time_minor,
						  gint run_time_micro)
{
	gchar *build_time_version;
	gchar *run_time_version;

	build_time_version = g_strdup_printf("%d.%d.%d",
										 build_time_major,
										 build_time_minor,
										 build_time_micro);
	run_time_version = g_strdup_printf("%d.%d.%d",
									   run_time_major,
									   run_time_minor,
									   run_time_micro);

	/* show versions of libraries used by GIMP */
	g_print(_("Using %s version %s (compiled against version %s)"),
			package, run_time_version, build_time_version);
	g_print("\n");

	g_free(run_time_version);
	g_free(build_time_version);
}

void help_show_version(void)
{
	int i;

	printf("\n");
	printf(_("Dibuja version %s"), PACKAGE_VERSION);
	printf("\n");
	printf("(c) %s ", COPY_YEAR);
	for(i = 0; authors[i] != NULL; i++){
		printf("%s ", authors[i]);
	}
	printf("\n%s\n\n", _(comments));
}

void help_print_info(void)
{
	gint major_version, minor_version, micro_version;

	printf(_("User interface xml file path:"));
	printf("\n");
	printf("  %s\n", UI_FILE);

	gegl_get_version(&major_version, &minor_version, &micro_version);

	gimp_show_library_version("GEGL", GEGL_MAJOR_VERSION, GEGL_MINOR_VERSION,
                              GEGL_MICRO_VERSION, major_version,
                              minor_version, micro_version);

	babl_get_version(&major_version, &minor_version, &micro_version);

	gimp_show_library_version("BABL", BABL_MAJOR_VERSION, BABL_MINOR_VERSION,
                              BABL_MICRO_VERSION, major_version,
                              minor_version, micro_version);

	gimp_show_library_version("GLib", GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION,
							  GLIB_MICRO_VERSION, glib_major_version,
							  glib_minor_version, glib_micro_version);

	gimp_show_library_version("GdkPixbuf", GDK_PIXBUF_MAJOR, GDK_PIXBUF_MINOR,
							  GDK_PIXBUF_MICRO, gdk_pixbuf_major_version,
							  gdk_pixbuf_minor_version, gdk_pixbuf_micro_version);

	gimp_show_library_version("GTK+", GTK_MAJOR_VERSION, GTK_MINOR_VERSION,
							  GTK_MICRO_VERSION, gtk_major_version,
							  gtk_minor_version, gtk_micro_version);
#ifdef HAVE_GEXIV2
	gimp_show_library_version("Gexiv2", GEXIV2_MAJOR_VERSION, GEXIV2_MINOR_VERSION,
							  GEXIV2_MICRO_VERSION, gexiv2_get_version() / 100 / 100,
							  gexiv2_get_version() / 100 % 100, gexiv2_get_version() % 100 );
#else
	printf(_("Not using the GExiv2 library"));
	printf("\n");
#endif
	puts("");
}




