/* 
 * (c) 2014 - 2023 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <locale.h>
#include <errno.h>
#include <string.h>

#include "config.h"
#include "dibuja.h"
#include "util.h"
#include "color.h"
#include "tools.h"
#include "file.h"
#include "undo.h"
#include "effects.h"
#include "menu.h"
#include "fx-dlg.h"
#include "settings.h"
#include "text.h"
#include "help.h"


#define show_widget(n)	(gtk_widget_show(get_widget((n))))

static GtkWidget *window;


GtkWindow *get_window(void)
{
	return GTK_WINDOW(window);
}

gboolean on_window1_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gboolean ret = TRUE;

	ret = do_quit();

	return ret;
}

GtkWidget *get_widget(const gchar *name)
{
	GtkWidget *w;

	w = GTK_WIDGET(g_object_get_data(G_OBJECT(window), name));
#ifdef WORKING_DIR
	if(!GTK_IS_WIDGET(w)){
		fprintf(stderr, "ERROR: \"%s\" is not a widget\n", name);
	}
#endif
	return w;
}

/* To fix combo problems in glade */
static void zoom_combo_init(GtkBuilder *builder)
{
	int i = 0, *index;
	char *text[] = {
		"10%",  "25%",  "33%",  "50%",  "67%",   "75%",   "100%",  "200%",
		"300%", "400%", "600%", "800%", "1000%", "1200%", "1600%",  NULL };
	GtkWidget *toolitem = 
					GTK_WIDGET(gtk_builder_get_object(builder, "toolitem1"));
	GtkWidget *combo = dib_combo_box_entry_new_text();
	GtkEntry *entry;

	for(i = 0; text[i] != NULL; i++){
		dib_combo_box_append_text(combo, text[i]);
	}

	gtk_container_add(GTK_CONTAINER(toolitem), combo);
	g_object_set_data(G_OBJECT(window), "combo", combo);
	g_signal_connect(G_OBJECT(combo), "changed", G_CALLBACK(zoom_combo_changed), NULL);
	dib_combo_box_set_active(combo, ZOOM_COMBO_100_PERC);
	index = g_malloc0(sizeof(int));
	*index = ZOOM_COMBO_100_PERC;
	g_object_set_data_full(G_OBJECT(combo), "index", (gpointer)index, g_free);

	entry = GTK_ENTRY(gtk_bin_get_child(GTK_BIN(combo)));
	gtk_entry_set_text(entry, "100%");
	gtk_entry_set_max_length(entry, 6);
	gtk_entry_set_width_chars(entry, 6);
	gtk_entry_set_alignment(entry, .5);
	g_signal_connect(G_OBJECT(entry), "activate",
					 G_CALLBACK(zoom_combo_entry_activate), NULL);
}

void fx_update_fx_menu(void)
{
	if(gegl_has_operation("gegl:grey") ||
		gegl_has_operation("gegl:gray")){
		show_widget("menu_effects_grayscale");
	}
	if(gegl_has_operation("gegl:invert") ||
		gegl_has_operation("gegl:invert-linear")){
		show_widget("menu_effects_invert");
	}
	if(gegl_has_operation("gegl:dibuja-insta-curve")){
		show_widget("menu_effects_insta_brannan");
		show_widget("menu_effects_insta_1977");
	}
	if(gegl_has_operation("gegl:pixelize")){
		show_widget("menu_effects_pixelate");
	}
	if(gegl_has_operation("gegl:brightness-contrast")){
		show_widget("menu_effects_brightness_contrast");
	}
	if(gegl_has_operation("gegl:gaussian-blur")){
		show_widget("menu_effects_gaussian_blur");
	}
	if(gegl_has_operation("gegl:hue-chroma")){
		show_widget("menu_effects_saturation_hue");
	}
	if(gegl_has_operation("gegl:posterize")){
		show_widget("menu_effects_posterize");
	}
	if(gegl_has_operation("gegl:threshold")){
		show_widget("menu_effects_threshold");
	}
	if(gegl_has_operation("gegl:unsharp-mask")){
		show_widget("menu_effects_sharpen");
	}
	if(gegl_has_operation("gegl:vignette")){
		show_widget("menu_effects_vignette");
	}
	if(gegl_has_operation("gegl:emboss")){
		show_widget("menu_effects_emboss");
	}
}

static gboolean fx_init_gegl(void)
{
	g_object_set(gegl_config(), "application-license", "GPL3", NULL);

	gegl_init(NULL, NULL);

	fx_update_fx_menu();

	return 1;
}

#if !GTK_CHECK_VERSION(3, 4, 0)
/* To fix setting widget markup problems in glade */
void on_realize(GtkWidget *widget, gpointer data)
{
	PangoFontDescription *font_desc;
	GtkWidget *tmp = widget;

	if(GTK_IS_BUTTON(tmp)){
		tmp = gtk_bin_get_child(GTK_BIN(widget));
		font_desc = pango_font_description_from_string("Sans 8");
	}
	else{
		font_desc = pango_font_description_from_string("Sans Bold 8");
	}
	gtk_widget_modify_font(tmp, font_desc);
	pango_font_description_free(font_desc);
}
#endif

int main(int argc, char *argv[])
{
	GtkWidget *alignment, *canvas, *w;

	GtkBuilder *builder = NULL;
	int ret = 0;
	GError *error = NULL;
	GtkAdjustment *adj;
	Settings settings = {0};
	char *filename;
	char **img_fname = NULL;
	int version = 0, info = 0;
	GOptionContext *optctxt = NULL;
	GOptionEntry entries[] = {
		{G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &img_fname, NULL, N_("[FILE]")},
		{"version", 0, 0, G_OPTION_ARG_NONE, &version, N_("Print the version"), NULL},
		{"info", 0, 0, G_OPTION_ARG_NONE, &info, N_("Print library information"), NULL},
		{NULL}
	};
	char *desc = NULL;
	gboolean gegl_is_init = FALSE;

#ifdef ENABLE_NLS
	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
	textdomain(GETTEXT_PACKAGE);
#endif

	desc = g_strdup(_("Simple image and photograph editor"));
	optctxt = g_option_context_new(NULL);
	g_option_context_set_translation_domain(optctxt, GETTEXT_PACKAGE);
	g_option_context_set_summary(optctxt, desc);
	g_free(desc);
	g_option_context_set_help_enabled(optctxt, TRUE);
    g_option_context_add_main_entries(optctxt, entries, NULL);
    g_option_context_add_group(optctxt, gtk_get_option_group(TRUE));
    g_option_context_parse(optctxt, &argc, &argv, &error);

	if(NULL != error){
		g_warning("%s\n", error->message);
		g_error_free(error);
	}

	g_option_context_free(optctxt);

	if(version || info){
		if(version){
			help_show_version();
		}
		if(info){
			if(!version)
				help_show_version();
			help_print_info();
		}
		return 0;
	}
	gtk_init(&argc, &argv);

	/* UI_FILE from util.h */
	builder = create_builder(UI_FILE, &error);
	if(error != NULL){
		g_message("Dibuja GtkBuilder error: %s.\n"
				  "Exiting...\n", error->message);
		message_box_error("Dibuja GtkBuilder error: %s\n", error->message);
		g_error_free(error);

		return 1;
	}

	gtk_icon_theme_append_search_path(gtk_icon_theme_get_default(), ICON_DIR_22x22);
	gtk_icon_theme_append_search_path(gtk_icon_theme_get_default(), ICON_DIR_48x48);

	window = builder_get_widget(builder, "window1");

	g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(gtk_main_quit), NULL);
	gtk_window_set_icon_from_file(GTK_WINDOW(window), DIBUJA_LOGO, &error);
	if(NULL != error){
		g_message("*** ERROR: %s\n", error->message);
		g_error_free(error);
	}

	filename = util_get_config_filename();

#ifndef WORKING_DIR
	char *config_dir =  util_get_dibuja_config_dir();

	if(!g_file_test(config_dir, G_FILE_TEST_IS_DIR)){
		int ret = g_mkdir_with_parents(config_dir, 0700);
		if(-1 == ret){
			fprintf(stderr, "ERROR: %s\n", strerror(errno));
		}
	}

	g_free(config_dir);
#endif

	settings_read(GTK_WINDOW(window), &settings, filename);
	g_free(filename);

	gtk_window_set_default_size(GTK_WINDOW(window),
								settings.width,
								settings.height);

	canvas = dibuja_create_canvas(builder, CANVAS_DEF_WIDTH, CANVAS_DEF_HEIGHT);

	alignment = builder_get_widget(builder, "alignment2");
	gtk_container_add(GTK_CONTAINER(alignment), canvas);

//	w = builder_get_widget(builder, "fontbutton1");
//	gtk_font_button_set_font_name(GTK_FONT_BUTTON(w), settings.fontname);

	switch(settings.fill_mode){
		case SC_FILL_NONE:
			w = builder_get_widget(builder, "rb_no_fill");
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
			break;
		case SC_FILL_FORE:
			w = builder_get_widget(builder, "rb_fore_fill");
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
			break;
		case SC_FILL_BACK:
			w = builder_get_widget(builder, "rb_back_fill");
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
			break;
		default:
			break;
	}

	// Set antialias
	w = builder_get_widget(builder, "check_button_antialias");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), settings.antialias);

	zoom_combo_init(builder);

	util_save_widgets(window, builder);

	w = builder_get_widget(builder, "dialog_text_vbox");
	text_init(w, settings.fontname);

	color_init(builder, settings.fgcolor, settings.bgcolor);

	tool_init(builder);

	// Set line width & use arrow
	adj = GTK_ADJUSTMENT(gtk_builder_get_object(builder, "adj_line_width"));
	gtk_adjustment_set_value(adj, settings.line_width);
	w = get_widget("check_button_use_arrow");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), settings.use_arrow);

#if 0
	// Selection
	w = get_widget("check_button_transparent_selection");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), settings.trans_sel);
#endif
	// Create undo
	undo_init(SIMPLE_CANVAS(canvas));

	gegl_is_init = fx_init_gegl();

	gtk_widget_show_all(window);

#ifdef HAVE_GEXIV2

extern void menu_file_info_activate(GtkWidget *widget, gpointer data);

	w = builder_get_widget(builder, "menu_file_info");
	g_signal_connect(G_OBJECT(w), "activate",
					 G_CALLBACK(menu_file_info_activate), NULL);
	gtk_widget_show(w);
	w = builder_get_widget(builder, "separatormenuitem7");
	gtk_widget_show(w);

#endif //HAVE_GEXIV2

	canvas_set_cursor(canvas, DIBUJA_CURSOR);

	// Checker board background
	w = get_widget("checkerb_bg");
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w),
								   settings.checkerb_bg);

	w = get_widget("cb_leave_cutout");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), settings.leave_cutout);

	gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(w));

	// Open file here 'cause we need the widget realized
	if(img_fname){
		ret = file_open_pixbuf(img_fname[0]);
	}

	if(!ret){
		dibuja_new_file(NULL, NULL, CANVAS_DEF_WIDTH, CANVAS_DEF_HEIGHT);
	}

	if(GTK_IS_BUILDER(builder))
		g_object_unref(builder);

	gtk_main();

	if(gegl_is_init)
		gegl_exit();

	return 0;
}
