/* 
 * (c) 2014 - 2023 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "metadata.h"
#include <string.h>
#include <glib/gi18n.h>

typedef struct{
	const char *key;
	const char *label;
}ExifData;

static ExifData exif_data[] = {
        //{ "Exif.Image.UniqueCameraModel", N_("Camera make")},
        //{ "Exif.Image.LocalizedCameraModel", N_("Camera model")},
        { "Exif.Image.Make"				,	N_("Camera make")		},
		{ "Exif.Image.Model"			,	N_("Camera model")		},
        { "Exif.Photo.DateTimeOriginal"	,	N_("Date Taken")		},
        { "Exif.Photo.ExposureTime"		,	N_("Exposure time")		},
        { "Exif.Photo.ExposureMode"		,	N_("Exposure mode")		},
        { "Exif.Photo.ApertureValue"	,	N_("Aperture Value")	},
        { "Exif.Photo.ISOSpeedRatings"	,	N_("ISO speed")			},
        { "Exif.Photo.Flash"			,	N_("Flash")				},
        { "Exif.Photo.MeteringMode"		,	N_("Metering mode")		},
        { "Exif.Image.FocalLength"		,	N_("Focal length")		},
        { "Exif.Image.Software"			,	N_("Software")			}
    };

static ExifData iptc_data[] = {
        { "Iptc.Application2.Caption"	,	N_("Description")		},
        { "Iptc.Application2.Copyright"	,	N_("Copyright")			},
        { "Iptc.Application2.Byline"	,	N_("Author")			}
    };

void metadata_read(const char *filename, MetadataCallback callback, GtkWidget *table, gint *row)
{
	GExiv2Metadata *meta = NULL;
	GError *error = NULL;
	int i;
	char *value = NULL;

	meta = gexiv2_metadata_new();
	if(!GEXIV2_IS_METADATA(meta)){
		fprintf(stderr, "[%s - %s(): %d] !GEXIV2_IS_METADATA(em->meta)\n",
				  __FILE__, __func__, __LINE__);
		
		return;
	}

	gexiv2_metadata_open_path(meta, filename, &error);
	if(error != NULL){
		fprintf(stderr, "ERROR: %s\n", error->message);
		g_error_free(error);
		g_object_unref(meta);

		return;
	}

	if(gexiv2_metadata_has_exif(meta)){
		for(i = 0; i < G_N_ELEMENTS(exif_data); i++){
#if GEXIV2_CHECK_VERSION(0, 12, 2)
			value = gexiv2_metadata_try_get_tag_interpreted_string(meta, exif_data[i].key, NULL);
#else
			value = gexiv2_metadata_get_tag_interpreted_string(meta, exif_data[i].key);
#endif
			if(value){
				callback(table, row, exif_data[i].label, value);
			}
		}
	}

	if(gexiv2_metadata_has_iptc(meta)){
		for(i = 0; i < G_N_ELEMENTS(iptc_data); i++){
#if GEXIV2_CHECK_VERSION(0, 12, 2)
			value = gexiv2_metadata_try_get_tag_interpreted_string(meta, iptc_data[i].key, NULL);
#else
			value = gexiv2_metadata_get_tag_interpreted_string(meta, iptc_data[i].key);
#endif
			if(value){
				callback(table, row, iptc_data[i].label, value);
			}
		}
	}

	g_object_unref(meta);
}


GExiv2Metadata *metatada_get(const char *filename)
{
	GExiv2Metadata *meta = NULL;
	GError *error = NULL;

	meta = gexiv2_metadata_new();
	if(!GEXIV2_IS_METADATA(meta)){
		fprintf(stderr, "[%s - %s(): %d] !GEXIV2_IS_METADATA(em->meta)\n",
				  __FILE__, __func__, __LINE__);
		return NULL;
	}

	gexiv2_metadata_open_path(meta, filename, &error);
	if(error != NULL){
		fprintf(stderr, "ERROR: %s\n", error->message);
		g_error_free(error);

		g_object_unref(meta);
		meta = NULL;
	}

	return meta;
}

void metadata_save(GExiv2Metadata *meta, const char *filename)
{
	GError *error = NULL;

	if(!gexiv2_metadata_save_file(meta, filename, &error)){
		fprintf(stderr, "ERROR: %s\n", error->message);
		g_error_free(error);
	}	
}

gboolean metadata_can_write_to(char *format)
{
	char *formats[] = {
		"jpeg", "png", "tiff", "webm", NULL
	};

	for(int i = 0; formats[i] != NULL; i++){
		if(!strcmp(formats[i], format)){
			return TRUE;
		}
	}

	return FALSE;
}
