/* 
 * (c) 2014 - 2023 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>
#include "tools.h"
#include "polyline.h"
#include "dibuja.h"
#include "util.h"
#include "draw-funcs.h"
#include "undo.h"
#include "file.h"


gboolean polyline_button_press(GtkWidget *widget, GdkEventButton *event, gpointer data);
gboolean polyline_motion(GtkWidget *widget, GdkEventMotion *event, gpointer data);
gboolean polyline_button_release(GtkWidget *widget, GdkEventButton *event, gpointer data);
gboolean polyline_expose(GtkWidget *widget, cairo_t *cr, gpointer data);
static void polyline_finish(void);
static gboolean polyline_cancel(void);

static DrawTool tool = {
				polyline_button_press,
				polyline_motion,
				polyline_button_release,
				polyline_expose,
				polyline_finish,
				polyline_button_press,
				polyline_cancel
			};

static GArray *lines = NULL;
static gboolean do_polygon = TRUE;

DrawTool *polyline_get_tool(void)
{
	do_polygon = FALSE;
	return &tool;
}

DrawTool *polygon_get_tool(void)
{
	do_polygon = TRUE;
	return &tool;
}

void set_bg_color(cairo_t *cr)
{
	SCColor bg;
	dibuja_get_bg_sccolor(&bg);
	cairo_set_source_rgba(cr, bg.r, bg.g, bg.b, bg.a);
}

void set_fg_color(cairo_t *cr)
{
	SCColor fg;
	dibuja_get_fg_sccolor(&fg);
	cairo_set_source_rgba(cr, fg.r, fg.g, fg.b, fg.a);
}

static gint poly_get_array_last(GArray *array)
{
	g_return_val_if_fail(array != NULL, 0);

	if(0 == array->len) return 0;

	return array->len - 1;
}
#if 0
static void polyline_calc_update_area(GArray *garray, GdkRectangle *r)
{
	GdkPoint pt;
	int i, maxx, maxy, minx, miny;
	double z = dibuja_get_zoom();
	int line_width = dibuja_get_line_width();

	if(!garray || !r)
		return;

	pt = g_array_index(garray, GdkPoint, 0);

	maxx = minx = pt.x;
	maxy = miny = pt.y;

	for(i = 1; i < garray->len; i++){
		pt = g_array_index(garray, GdkPoint, i);
		minx = MIN(minx, pt.x);
		miny = MIN(miny, pt.y);
		maxx = MAX(maxx, pt.x);
		maxy = MAX(maxy, pt.y);
	}

	set_update_rect(r, minx, miny, maxx, maxy, z);
	resize_rect(r, (line_width * 2) * z, (line_width * 2) * z);
}
#endif
static void polyline_add(int x, int y)
{
	GdkPoint pt = {x, y};

	g_array_append_val(lines, pt);
}

static gboolean draw_poly(cairo_t *cr, GArray *garray)
{
	guint i;
	GdkPoint pt;

	if(!garray) return FALSE;

	pt = g_array_index(garray, GdkPoint, 0);

	cairo_move_to(cr, (double)pt.x, (double)pt.y);

	for(i = 1; i < garray->len; i++){
		pt = g_array_index(garray, GdkPoint, i);
		cairo_line_to(cr, (double)pt.x, (double)pt.y);
	}

	return TRUE;
}

static void draw_poly_init(cairo_t *cr, GArray *garray)
{
	int ret;

	ret = draw_poly(cr, garray);
	if(!ret) return;

	cairo_stroke(cr);
}

static void draw_poly_canvas(cairo_t *cr, GArray *garray, gboolean do_polygon)
{
	int ret;
	GdkRectangle area;
	double xa, ya, xb, yb;
	ShapeFillType fill_mode = dibuja_get_fill_mode();

	ret = draw_poly(cr, garray);
	if(!ret) return;

	if(do_polygon){
		cairo_close_path(cr);
	}

	cairo_stroke_extents(cr, &xa, &ya, &xb, &yb);
	rect_from_pts(&area, (int)xa, (int)ya, (int)xb, (int)yb);
	resize_rect(&area, dibuja_get_line_width(), dibuja_get_line_width());

	if(do_polygon)
		undo_add(SIMPLE_CANVAS(dibuja_get_canvas()), area, _("Polygon"), UNDO_TYPE_POLYGON);
	else
		undo_add(SIMPLE_CANVAS(dibuja_get_canvas()), area, _("Polyline"), UNDO_TYPE_POLYLINE);

	if(do_polygon && SHAPE_FILL_NONE != fill_mode){
		if(SHAPE_FILL_BACK == fill_mode){
			set_bg_color(cr);
		}
		else{
			set_fg_color(cr);
		}

		cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
		cairo_fill_preserve(cr);
	}

	set_fg_color(cr);
	cairo_stroke(cr);
}

static void polyline_delete(void)
{
	if(lines)
		g_array_free(lines, TRUE);

	lines = NULL;
}

static gboolean polyline_cancel(void)
{
	polyline_delete();
	gtk_widget_queue_draw(dibuja_get_canvas());

	return FALSE;
}

static void polyline_finish(void)
{
	if(lines){
		cairo_t *cr = dibuja_get_cr(CR_SURFACE);

		cairo_set_line_width(cr, (double)dibuja_get_line_width());
		// Destroy array & draw to surface
		draw_poly_canvas(cr, lines, do_polygon);
		cairo_destroy(cr);
		polyline_delete();
		gtk_widget_queue_draw(dibuja_get_canvas());
	}
}

gboolean
polyline_button_press(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	if(3 == event->button){

		polyline_finish();

		return TRUE;
	}
	else if(1 != event->button){
		return TRUE;
	}

	if(!lines){
		lines = g_array_new(FALSE, FALSE, sizeof(GdkPoint));
		polyline_add((int)event->x, (int)event->y);
	}

	polyline_add((int)event->x, (int)event->y);
	gtk_widget_queue_draw(widget);

	return TRUE;
}

gboolean polyline_motion(GtkWidget *widget, GdkEventMotion *event, gpointer data)
{
	if(event->state & GDK_BUTTON1_MASK && lines != NULL){
		GdkPoint *pt2;
		int n = poly_get_array_last(lines);

		pt2 = &g_array_index(lines, GdkPoint, n);
		pt2->x = (int)event->x;
		pt2->y = (int)event->y;

		if(event->state & GDK_CONTROL_MASK){
			int prev = n - 1;
			if(prev >= 0){
				GdkPoint *pt1 = &g_array_index(lines, GdkPoint, prev);

				if(pt1){
					double x = (double)(pt2->x), y = (double)(pt2->y);

					draw_constrain_pt(pt1->x, pt1->y, pt2->x, pt2->y, &x, &y);
					pt2->x = x;
					pt2->y = y;
				}
			}
		}

		gtk_widget_queue_draw(widget);
	}

	return TRUE;
}

gboolean
polyline_button_release(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	GdkPoint *pt2;
	int n;

	if(!lines) return TRUE;
	if(1 != event->button) return TRUE;

	n = poly_get_array_last(lines);
	pt2 = &g_array_index(lines, GdkPoint, n);
	pt2->x = (int)event->x;
	pt2->y = (int)event->y;

	if(event->state & GDK_CONTROL_MASK){
		int prev = n - 1;
		if(prev >= 0){
			GdkPoint *pt1 = &g_array_index(lines, GdkPoint, prev);

			if(pt1){
				double x = (double)(pt2->x), y = (double)(pt2->y);

				draw_constrain_pt(pt1->x, pt1->y, pt2->x, pt2->y, &x, &y);
				pt2->x = x;
				pt2->y = y;
			}
		}
	}

	gtk_widget_queue_draw(widget);

	return TRUE;
}

gboolean polyline_expose(GtkWidget *widget, cairo_t *cr, gpointer data)
{
	draw_poly_init(cr, lines);

	return TRUE;
}

