/* 
 * (c) 2014 - 2023 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>
#include <ctype.h>
#include <stdlib.h>
#include "dibuja.h"
#include "util.h"
#include "selection.h"

#define DEF_PERC	"200%"

static GtkWidget *scale_dialog = NULL;
static int origw, origh;
static GtkWidget *wspin = NULL, *hspin = NULL;
static gboolean keep_aspect = TRUE;
static GtkWidget *scale_vbox2 = NULL, *scale_vbox3 = NULL;
static double combo_scale = 1.0;

void dialog_scale_checkbutton_toggled(GtkToggleButton *scale_checkbutton, gpointer user_data)
{
	keep_aspect = gtk_toggle_button_get_active(scale_checkbutton);
}

void dialog_scale_spinbutton_width_value_changed(GtkSpinButton *spin, gpointer data)
{
	double val;
	gboolean focus = 0;

	focus = gtk_widget_has_focus(GTK_WIDGET(spin));
	val = gtk_spin_button_get_value(spin);

	if(focus && keep_aspect)
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(hspin), val * origh / origw + .5);
}

void dialog_scale_spinbutton_height_value_changed(GtkSpinButton *spin, gpointer data)
{
	double val;
	gboolean focus = 0;

	focus = gtk_widget_has_focus(GTK_WIDGET(spin));
	val = gtk_spin_button_get_value(spin);

	if(focus && keep_aspect)
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(wspin), val * origw / origh + .5);
}

/**** Combo code **************************************************************/

static int combo_get_scale(int n)
{
	return (int)((combo_scale / 100.) * (double)n + .5);
}

static void enable_children(GtkWidget *widget, gpointer data)
{
	gboolean *sensitive = (gboolean *)data;

	if(GTK_IS_WIDGET(widget) && sensitive){
		gtk_widget_set_sensitive(widget, *sensitive);
	}
}

static void rb_scale_toggled(GtkToggleButton *button, gpointer data)
{
	GtkContainer *container = GTK_CONTAINER(data);
	gboolean active;

	active = gtk_toggle_button_get_active(button);
	gtk_container_foreach(container, enable_children, (gpointer)&active);
}

static void label_percent_set(char *str_scale)
{
	GtkLabel *label = GTK_LABEL(get_widget("label_percent_scale"));

	if(str_scale){
		char *copy;
		gint len = (gint)strlen(str_scale);
		gint i = 0, j = 0;
		double tmp;

		copy = g_malloc0((gsize)len + 1);

		/* Filter out non digits */
		for(i = 0, j = 0; i < len; i++){
			if(isdigit(str_scale[i])){
				copy[j] = str_scale[i];
				j++;
			}
			else
				/* Stop at first non digit,
				   will be cut off at atoi */
				break;
		}

		tmp = (double)atoi(copy);

		if((int)tmp <= 0){ // TODO: Keep image to sane size
			g_warning("Image %d scaling out of range.", (int)tmp);
		}
		else{
			char *text;

			combo_scale = tmp;
			text = g_strdup_printf(_("New Size: %d x %d"),
									combo_get_scale(origw),
									combo_get_scale(origh));

			gtk_label_set_markup(label, text);
			g_free(text);
		}
		g_free(copy);
	}
}

static int *combo_get_index(GtkWidget *cbox)
{
	return (int *)(g_object_get_data(G_OBJECT(cbox), "index"));
}

static void combo_set_index(GtkWidget *cbox, int index)
{
	if(GTK_IS_WIDGET(cbox)){
		int *i = combo_get_index(cbox);
		if(i){
			*i = index;
		}
	}
}

static void scale_combo_changed(GtkWidget *cbox)
{
	int active;
	gchar *text;

	active = dib_combo_box_get_active(cbox);
	if(active < 0)return;

	text = dib_combo_box_get_active_text(cbox);
	combo_set_index(GTK_WIDGET(cbox), active);

	if(text){
		label_percent_set(text);
		g_free(text);
	}
}

static void scale_combo_entry_activate(GtkEntry *entry)
{
	char *text = g_strdup(gtk_entry_get_text(entry));

	if(text){
		label_percent_set(text);
		g_free(text);
	}
}

static GtkWidget *create_combo()
{
	GtkWidget *cbox;
	GtkEntry *entry;
	int i = 0;
	int percent[15] = { 10,  20,  33,  50,  75,  90,  100,
					   150, 200, 300, 400, 600, 800, 1000, 0};
	char text[9] = "";

	cbox = dib_combo_box_entry_new_text();

	for(i = 0; percent[i] != 0; i++){
		g_snprintf(text, sizeof(text), "%d%%", percent[i]);
		dib_combo_box_append_text(cbox, text);
	}

	g_signal_connect(G_OBJECT(cbox), "changed",
					 G_CALLBACK(scale_combo_changed), NULL);	

	entry = GTK_ENTRY(gtk_bin_get_child(GTK_BIN(cbox)));
	gtk_entry_set_text(entry, DEF_PERC);
	gtk_entry_set_max_length(entry, 6);
	gtk_entry_set_width_chars(entry, 6);
	gtk_entry_set_alignment(entry, .5);
	g_signal_connect(G_OBJECT(entry), "activate",
					 G_CALLBACK(scale_combo_entry_activate), NULL);

	return cbox;
}
/**** End combo code **********************************************************/

void dialog_scale_run(void)
{
	gint ret = 0;
	GtkWidget *rb_dimensions, *rb_percentage;

	GtkWidget *widget;
	char *str;
	int neww, newh;

	if(sel_have()){
		cairo_surface_t *surf = sel_get_cur_image();
		origw = cairo_image_surface_get_width(surf);
		origh = cairo_image_surface_get_height(surf);
	}
	else{
		origw = dibuja_get_width();
		origh = dibuja_get_height();
	}	

	if(!GTK_IS_WIDGET(scale_dialog)){
		GtkWidget *combo;
		gboolean sensitive = FALSE;

		scale_dialog  = get_widget("dialog_scale" );
		rb_dimensions = get_widget("rb_dimensions");
		rb_percentage = get_widget("rb_percentage");

		scale_vbox2   = get_widget("scale_vbox2"  );
		scale_vbox3   = get_widget("scale_vbox3"  );

		g_signal_connect(G_OBJECT(rb_percentage), "toggled",
						 G_CALLBACK(rb_scale_toggled),
						 (gpointer)scale_vbox3);

		g_signal_connect(G_OBJECT(rb_dimensions), "toggled",
						 G_CALLBACK(rb_scale_toggled),
						 (gpointer)scale_vbox2);

		combo = create_combo();
		gtk_box_pack_start(GTK_BOX(scale_vbox3), combo, FALSE, TRUE, 2);
		gtk_widget_show(combo);

		sensitive = TRUE;
		gtk_container_foreach(GTK_CONTAINER(scale_vbox3), enable_children,
							  (gpointer)&sensitive);
		sensitive = FALSE;
		gtk_container_foreach(GTK_CONTAINER(scale_vbox2), enable_children,
							  (gpointer)&sensitive);

		util_dialog_def_size(scale_dialog);

		label_percent_set(DEF_PERC);
	}

	widget = get_widget("dialog_scale_label_size");
	str = g_strdup_printf(_("<b>Current Size: %d x %d</b>"), origw, origh);
	gtk_label_set_markup(GTK_LABEL(widget), str);
	g_free(str);

	wspin = get_widget("dialog_scale_spinbutton_width");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(wspin), (double)origw);
	hspin = get_widget("dialog_scale_spinbutton_height");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(hspin), (double)origh);

	str = g_strdup_printf("%d", (int)combo_scale);
	label_percent_set(str);
	g_free(str);

	ret = gtk_dialog_run (GTK_DIALOG (scale_dialog));
	switch (ret){
    	case GTK_RESPONSE_APPLY:
    		rb_percentage = get_widget("rb_percentage");
    		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rb_percentage))){
    			neww = combo_get_scale(origw);
    			newh = combo_get_scale(origh);
    		}
    		else{
    			neww = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(wspin));
    			newh = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(hspin));
    		}

    		if(sel_have()){
    			sel_scale(neww, newh);
    		}
    		else{
    			dibuja_scale_image(neww, newh);
    		}

    		break;
    	default:
    		break;
	}

	gtk_widget_hide (scale_dialog);
}

