/* 
 * (c) 2023 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <math.h>
#include "sel-frame.h"
#include "draw-funcs.h"


struct _SelFrame{
	GArray *garray;
	SelFrameType sel_type;
	guint state; // Only for SEL_FRAME_ELLIP & SEL_FRAME_RECT
};


static void sel_frame_get_crtl_rect(GdkRectangle *r,
                                    double sx, double sy,
                                    double ex, double ey)
{
	double len = sqrt(((sx - ex) * (sx - ex)) + ((sy - ey) * (sy - ey)));

	rect_from_pts(r, (int)roundf(sx - len), (int)roundf(sy - len),
				  (int)roundf(sx + len), (int)roundf(sy + len) );
}

static void sel_draw_ellipse(cairo_t *cr, GdkRectangle r)
{
	gdouble x = (gdouble)r.x, y = (gdouble)r.y,
	        w = (gdouble)r.width, h = (gdouble)r.height;

	cairo_save (cr);
	cairo_translate (cr, x + w / 2., y + h / 2.);
	cairo_scale (cr, w / 2., h / 2.);
	cairo_arc (cr, 0., 0., 1., 0., 2 * M_PI);
	cairo_restore (cr);
}

static void sel_frame_draw_sel(SelFrame *sel_frame, cairo_t *cr)
{
	SelFrameType type;
	GdkRectangle r;

	g_return_if_fail(sel_frame != NULL);

	type = sel_frame_get_type(sel_frame);

	if(SEL_FRAME_FREE == type){
		GdkPoint pt;
		guint i;
		if(sel_frame->garray->len < 2)
			return;

		pt = g_array_index(sel_frame->garray, GdkPoint, 0);
		cairo_move_to(cr, (double)pt.x, (double)pt.y);
		for(i = 1; i < sel_frame->garray->len; i++){
			pt = g_array_index(sel_frame->garray, GdkPoint, i);
			cairo_line_to(cr, (double)pt.x, (double)pt.y);
		}

		cairo_stroke(cr);
		return;
	}

	sel_frame_get_area(sel_frame, &r);

	if(SEL_FRAME_ELLIP == type){
			sel_draw_ellipse(cr, r);
	}
	else if(SEL_FRAME_RECT == type){
		cairo_save(cr);
		cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
		cairo_rectangle(cr, (double)r.x, (double)r.y,
							(double)r.width, (double)r.height);
		cairo_restore(cr);
	}
	else
		return;

	cairo_stroke(cr);
}

void sel_frame_draw(SelFrame *sel_frame, cairo_t *cr)
{
	double dashes[2] = {2., 2.};
	int num_dashes = (int)(sizeof(dashes) / sizeof(dashes[0]));

	cairo_save(cr);

	cairo_set_line_width(cr, 1.0);
	cairo_set_source_rgba(cr, 1., 1., 1., 1.);

	sel_frame_draw_sel(sel_frame, cr);

	cairo_set_source_rgba(cr, 0., 0., 0., 1.);
	cairo_set_dash(cr, dashes, num_dashes, 0.);

	sel_frame_draw_sel(sel_frame, cr);

	cairo_restore(cr);
}

SelFrame *sel_frame_create(SelFrameType sel_type, int x, int y)
{
	SelFrame *sel_frame;
	gboolean zero_terminated = FALSE;

	if(!(sel_type >= SEL_FRAME_RECT && SEL_FRAME_RECT < SEL_FRAME_LAST))
		sel_type = SEL_FRAME_RECT;

	sel_frame = g_malloc0(sizeof(SelFrame));
	if(!sel_frame) return NULL;

	sel_frame->sel_type = sel_type;
	sel_frame->garray = g_array_new(zero_terminated, FALSE, sizeof(GdkPoint));
	if(SEL_FRAME_FREE == sel_frame->sel_type)
		sel_frame_add_point(sel_frame, x, y, 0);
	else{
		GdkPoint pt = {x, y};

		g_array_append_val(sel_frame->garray, pt);
		g_array_append_val(sel_frame->garray, pt);
	}

	return sel_frame;
}

void sel_frame_destroy(SelFrame *sel_frame)
{
	if(sel_frame){
		g_array_free(sel_frame->garray, TRUE);
		sel_frame->garray = NULL;
		g_free(sel_frame);
		sel_frame = NULL;
	}
}

void sel_frame_get_area(SelFrame *sel_frame, GdkRectangle *r)
{
	g_return_if_fail(sel_frame != NULL);
	g_return_if_fail(sel_frame->garray != NULL);
	g_return_if_fail(r != NULL);

	if(SEL_FRAME_FREE == sel_frame->sel_type){
		GdkPoint pt;
		int x1, y1, x2, y2;
		guint i = 0;

		pt = g_array_index(sel_frame->garray, GdkPoint, 0);
		x1 = x2 = pt.x;
		y1 = y2 = pt.y;

		for(i = 0; i < sel_frame->garray->len; i++){
			pt = g_array_index(sel_frame->garray, GdkPoint, i);

			if(pt.x < x1) x1 = pt.x;
			if(pt.y < y1) y1 = pt.y;
			if(pt.x > x2) x2 = pt.x;
			if(pt.y > y2) y2 = pt.y;
		}

		rect_from_pts(r, x1, y1, x2, y2);
	}
	else{
		GdkPoint *pt1, *pt2;
		double sx, sy, ex, ey;

		pt1 = &g_array_index(sel_frame->garray, GdkPoint, 0);
		pt2 = &g_array_index(sel_frame->garray, GdkPoint, 1);

		if(!(pt1 != NULL && pt2 != NULL))
			return;

		sx = (double)pt1->x;     sy = (double)pt1->y;
		ex = (double)pt2->x;     ey = (double)pt2->y;

		if(sel_frame->state & GDK_CONTROL_MASK){
			sel_frame_get_crtl_rect(r, sx, sy, ex, ey);
		}
		else if(sel_frame->state & GDK_SHIFT_MASK){
			draw_constrain_pt(sx, sy, ex, ey, &ex, &ey);
			rect_from_pts(r, (int)sx, (int)sy, (int)ex, (int)ey);
		}
		else{
			rect_from_pts(r, (int)sx, (int)sy, (int)ex, (int)ey);
		}
	}
}
#if 0
void sel_frame_get_area(SelFrame *sel_frame, GdkRectangle *r)
{
	GdkPoint pt;
	int x1, y1, x2, y2;

	g_return_if_fail(sel_frame != NULL);
	g_return_if_fail(r != NULL);

	if(SEL_FRAME_FREE == sel_frame->sel_type){
		guint i = 0;

		pt = g_array_index(sel_frame->garray, GdkPoint, 0);
		x1 = x2 = pt.x;
		y1 = y2 = pt.y;

		for(i = 0; i < sel_frame->garray->len; i++){
			pt = g_array_index(sel_frame->garray, GdkPoint, i);

			if(pt.x < x1) x1 = pt.x;
			if(pt.y < y1) y1 = pt.y;
			if(pt.x > x2) x2 = pt.x;
			if(pt.y > y2) y2 = pt.y;
		}

		rect_from_pts(r, x1, y1, x2, y2);
	}
	else{

		if(sel_frame->state & GDK_CONTROL_MASK){
			GdkPoint *pt1, *pt2;

			pt1 = &g_array_index(sel_frame->garray, GdkPoint, 0);
			pt2 = &g_array_index(sel_frame->garray, GdkPoint, 1);

			if(!(pt1 != NULL && pt2 != NULL))
				return;

			sel_frame_get_crtl_rect(r, (double)pt1->x, (double)pt1->y,
    	                                (double)pt2->x, (double)pt2->y);
		}
		else if(sel_frame->state & GDK_SHIFT_MASK){
			double sx, sy, ex, ey;
			GdkPoint *pt1, *pt2;

			pt1 = &g_array_index(sel_frame->garray, GdkPoint, 0);
			pt2 = &g_array_index(sel_frame->garray, GdkPoint, 1);

			if(!(pt1 != NULL && pt2 != NULL))
				return;

			sx = (double)pt1->x;     sy = (double)pt1->y;
			ex = (double)pt2->x;     ey = (double)pt2->y;

			draw_constrain_pt(sx, sy, ex, ey, &ex, &ey);
			rect_from_pts(r, (int)sx, (int)sy, (int)ex, (int)ey);
		}
		else{
			pt = g_array_index(sel_frame->garray, GdkPoint, 0);
			x1 = pt.x ; y1 = pt.y;
			pt = g_array_index(sel_frame->garray, GdkPoint, 1);
			x2 = pt.x ; y2 = pt.y;
			rect_from_pts(r, x1, y1, x2, y2);
		}

	}
}
#endif
void sel_frame_add_point(SelFrame *sel_frame, int x, int y, guint state)
{
	g_return_if_fail(sel_frame != NULL);

	sel_frame->state = state;

	if(SEL_FRAME_FREE == sel_frame->sel_type){
		GdkPoint pt = {x, y};

		g_array_append_val(sel_frame->garray, pt);
	}
	else{
		GdkPoint *pt;

		pt = &g_array_index(sel_frame->garray, GdkPoint, 1);
		pt->x = x;
		pt->y = y;
	}
}

SelFrameType sel_frame_get_type(SelFrame *sel_frame)
{
	g_return_val_if_fail(sel_frame != NULL, SEL_FRAME_NONE);

	return sel_frame->sel_type;
}

GArray *sel_frame_get_pts(SelFrame *sel_frame)
{
	GArray *points = NULL;
	GdkPoint p;

	g_return_val_if_fail(sel_frame != NULL, NULL);

	if(SEL_FRAME_FREE != sel_frame->sel_type){
		GdkRectangle r;

		sel_frame_get_area(sel_frame, &r);

		points = g_array_new(FALSE, FALSE, sizeof(GdkPoint));

		p.x = r.x;   p.y = r.y;
		g_array_append_val(points, p);

		p.x = r.x + r.width;   p.y = r.y;
		g_array_append_val(points, p);

		p.x = r.x + r.width;   p.y = r.y + r.height;
		g_array_append_val(points, p);

		p.x = r.x;   p.y = r.y + r.height;
		g_array_append_val(points, p);

		return points;
	}

	guint i;

	points = g_array_new(FALSE, FALSE, sizeof(GdkPoint));
	for(i = 0; i < sel_frame->garray->len; i++){
		p = g_array_index(sel_frame->garray, GdkPoint, i);
		g_array_append_val(points, p);
	}

	return points;
}
