/*
 * sel-handle.c
 * Copyright (C) 2012 - 2023 Juan Balderas <juan.balderas.0@gmail.com>
 * 
 * sel-handle.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * sel-handle.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "sel-handle.h"


struct _SelHandlePrivate
{
	gint x, y;
	guint width, height;
	gboolean active;
	SelHandleType type;
	SCColor color;
};


#if GLIB_CHECK_VERSION(2, 58, 0)
  #define sel_handle_get_instance_priv  sel_handle_get_instance_private
#else
  #define SEL_HANDLE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
                                       SEL_TYPE_HANDLE, SelHandlePrivate))

  #define sel_handle_get_instance_priv	SEL_HANDLE_GET_PRIVATE
#endif


G_DEFINE_TYPE_WITH_CODE (SelHandle, sel_handle, G_TYPE_OBJECT, G_ADD_PRIVATE(SelHandle))

static void
sel_handle_init (SelHandle *object)
{
	SelHandle *sh;
	SCColor color = {0., 0., 0., 1.};

	object->priv = sel_handle_get_instance_priv(object);

	sh = SEL_HANDLE(object);
	sh->priv->x       = 0;
	sh->priv->y       = 0;
	sh->priv->width   = DEF_HANDLE_WIDTH;
	sh->priv->height  = DEF_HANDLE_HEIGHT;
	sh->priv->active  = FALSE;
	sh->priv->type    = SEL_HANDLE_RECT;
	sh->priv->color   = color;
}

static void
sel_handle_finalize (GObject *object)
{
	G_OBJECT_CLASS (sel_handle_parent_class)->finalize (object);
}

static void
sel_handle_class_init (SelHandleClass *klass)
{
	GObjectClass* object_class = G_OBJECT_CLASS (klass);
#if 0
#if GLIB_CHECK_VERSION(2, 58, 0)
	G_ADD_PRIVATE(SelHandlePrivate);
#else
	g_type_class_add_private (klass, sizeof (SelHandlePrivate));
#endif
#endif

	object_class->finalize = sel_handle_finalize;
}


void
sel_handle_draw (SelHandle *sh, cairo_t *cr)
{
	double x, y, w, h;

	g_return_if_fail(cr != NULL);

	x = (double)sh->priv->x;
	y = (double)sh->priv->y;
	w = (double)sh->priv->width;
	h = (double)sh->priv->height;

	if(sel_handle_is_active(sh)){
		cairo_set_source_rgb(cr, 0., 1., 0.);
	}
	else{
		cairo_set_source_rgba(cr,
							  sh->priv->color.r,
							  sh->priv->color.g,
							  sh->priv->color.b,
							  sh->priv->color.a);
	}

	if(SEL_HANDLE_ROUND == sh->priv->type){
		cairo_save (cr);
		cairo_translate (cr, x + w / 2., y + h / 2.);
		cairo_scale (cr, w / 2., h / 2.);
		cairo_arc (cr, 0., 0., 1., 0., 2 * M_PI);
		cairo_restore (cr);
	}
	else{
		cairo_rectangle(cr, x, y, w, h);
	}
	cairo_fill(cr);
}

void
sel_handle_move (SelHandle *sh, gint x, gint y)
{
	if(SEL_IS_HANDLE(sh)){
		sh->priv->x = x;
		sh->priv->y = y;
	}
}

SelHandle *
sel_handle_new (void)
{
	return SEL_HANDLE(g_object_new (SEL_TYPE_HANDLE, NULL));
}

void
sel_handle_set_active (SelHandle *sh, gboolean active)
{
	sh->priv->active = active;
}

gboolean
sel_handle_is_active (SelHandle *sh)
{
	return sh->priv->active;
}

gboolean
sel_handle_pt_in (SelHandle *sh, gint x, gint y)
{
	g_return_val_if_fail(SEL_IS_HANDLE(sh), FALSE);
	if((x >= sh->priv->x) && (x <= sh->priv->x + (int)sh->priv->width)){
		if((y >= sh->priv->y) && (y <= sh->priv->y + (int)sh->priv->height)){
			return TRUE;
		}
	}
	return FALSE;
}

void
sel_handle_get_pos (SelHandle *sh, GdkPoint *pt)
{
	g_return_if_fail(pt != NULL);
	pt->x = sh->priv->x;
	pt->y = sh->priv->y;
}

void
sel_handle_set_color (SelHandle *sh, SCColor *color)
{
	g_return_if_fail(color != NULL);
	sh->priv->color = *color;
}

void sel_handle_set_type(SelHandle *sh, SelHandleType type)
{
	if(SEL_HANDLE_RECT == type){
		sh->priv->type = SEL_HANDLE_ROUND;
	}
	else{
		sh->priv->type = type;
	}
}

void sel_handle_set_dims(SelHandle *sh, guint w, guint h)
{
	sh->priv->width = w;
	sh->priv->height = h;
}

gboolean sel_handle_get_pt_offset(SelHandle *sh, gint *x, gint *y)
{
	g_return_val_if_fail(SEL_IS_HANDLE(sh), FALSE);
	g_return_val_if_fail(!(x == NULL) && !(y == NULL), FALSE);

	if(!sel_handle_pt_in(sh, *x, *y))
		return FALSE;

	*x -= sh->priv->x;
	*y -= sh->priv->y;

	return TRUE;
}

gint sel_handle_get_width (SelHandle *sh)
{
	return sh->priv->width;
}

gint sel_handle_get_height (SelHandle *sh)
{
	return sh->priv->height;
}







