/*
 * (c) 2015 - 2023 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <math.h>
#include "simple-canvas.h"

#if GLIB_CHECK_VERSION(2, 58, 0)
  #define simple_canvas_get_instance_priv  simple_canvas_get_instance_private
#else
  #define SIMPLE_CANVAS_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
                                          SIMPLE_TYPE_CANVAS,                 \
                                          SimpleCanvasPrivate))

  #define simple_canvas_get_instance_priv	SIMPLE_CANVAS_GET_PRIVATE
#endif

#define MAX_ZOOM (16.0)

//G_DEFINE_TYPE (SimpleCanvas, simple_canvas, GTK_TYPE_DRAWING_AREA)

struct _SimpleCanvasPrivate{
	cairo_surface_t *surface;
	cairo_surface_t *surf_tmp;
	GdkPoint press;
	GdkPoint release;
	double zoom;
	gboolean show_grid;
	GdkRectangle update_area;
	double max_zoom;

	int line_width;
	cairo_antialias_t aa;
	SCColor fg;
	SCColor bg;
	SCFillType fill_type;

	cairo_surface_t *background;
	gboolean checkerb_bg;
};

enum {
	SCANVAS_EXPOSE,
	SCANVAS_BUTTON_PRESS,
	SCANVAS_BUTTON_RELEASE,
	SCANVAS_POINTER_MOTION,
	SCANVAS_IMAGE_CHANGED,
	LAST_SIGNAL
};

G_DEFINE_TYPE_WITH_CODE (SimpleCanvas, simple_canvas, GTK_TYPE_DRAWING_AREA, G_ADD_PRIVATE(SimpleCanvas))

static guint signals[LAST_SIGNAL] = { 0 };


static void
simple_canvas_unrealize (GtkWidget *widget);

#if GTK_MAJOR_VERSION >= 3
static gboolean canvas_draw(GtkWidget *canvas, cairo_t *cr);
#else
static gint canvas_expose(GtkWidget *widget, 
	                            GdkEventExpose *event);
#endif
static gint canvas_button_press(GtkWidget *widget, GdkEventButton *event);
static gint canvas_motion(GtkWidget *widget, GdkEventMotion *event);
static gint canvas_button_release(GtkWidget *widget, GdkEventButton *event);


static cairo_surface_t *simple_canvas_create_background(void)
{
	cairo_surface_t *bg;
	double x, y;
	cairo_t *cr;
	double rect_size = 8.;
	double col = 213. / 255.;

	bg = cairo_image_surface_create(DEF_SURF_FORMAT, 16, 16);

	if(!bg){
		g_warning("[%s()] Error creating surface\n", __func__);
		return NULL;
	}

	cr = cairo_create(bg);
	cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);

	x = y = 0.;
	cairo_set_source_rgba(cr, 1., 1., 1., 1.);
	cairo_rectangle(cr, x, y, (double)rect_size, (double)rect_size);
	cairo_fill(cr);

	x += rect_size;
	cairo_set_source_rgba(cr, col, col, col, 1.);
	cairo_rectangle(cr, x, y, (double)rect_size, (double)rect_size);
	cairo_fill(cr);

	x = 0.; y += rect_size;
	cairo_set_source_rgba(cr, col, col, col, 1.);
	cairo_rectangle(cr, x, y, (double)rect_size, (double)rect_size);
	cairo_fill(cr);

	x += rect_size;
	cairo_set_source_rgba(cr, 1., 1., 1., 1.);
	cairo_rectangle(cr, x, y, (double)rect_size, (double)rect_size);
	cairo_fill(cr);

	cairo_destroy(cr);

	return bg;
}

static void
simple_canvas_class_init (SimpleCanvasClass *klass)
{
	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

	widget_class->unrealize = simple_canvas_unrealize;
#if GTK_MAJOR_VERSION >= 3
	widget_class->draw = canvas_draw;
#else
	widget_class->expose_event = canvas_expose;
#endif
	widget_class->button_press_event = canvas_button_press;
	widget_class->button_release_event = canvas_button_release;
	widget_class->motion_notify_event = canvas_motion;

	signals[SCANVAS_EXPOSE] =
		g_signal_new ("simple-canvas-expose",
					  G_OBJECT_CLASS_TYPE (widget_class),
					  G_SIGNAL_RUN_LAST,
					  G_STRUCT_OFFSET (SimpleCanvasClass, simple_canvas_expose),
					  NULL, NULL,
					  g_cclosure_marshal_VOID__POINTER,
					  G_TYPE_NONE,
					  1,
					  G_TYPE_POINTER );

	signals[SCANVAS_BUTTON_PRESS] =
		g_signal_new ("simple-canvas-button-press",
					  G_OBJECT_CLASS_TYPE (widget_class),
					  G_SIGNAL_RUN_LAST,
					  G_STRUCT_OFFSET (SimpleCanvasClass, simple_canvas_button_press),
					  NULL, NULL,
					  g_cclosure_marshal_VOID__POINTER,
					  G_TYPE_NONE,
					  1,
					  G_TYPE_POINTER );

	signals[SCANVAS_BUTTON_RELEASE] =
		g_signal_new ("simple-canvas-button-release",
					  G_OBJECT_CLASS_TYPE (widget_class),
					  G_SIGNAL_RUN_LAST,
					  G_STRUCT_OFFSET (SimpleCanvasClass, simple_canvas_button_release),
					  NULL, NULL,
					  g_cclosure_marshal_VOID__POINTER,
					  G_TYPE_NONE,
					  1,
					  G_TYPE_POINTER );

	signals[SCANVAS_POINTER_MOTION] =
		g_signal_new ("simple-canvas-pointer-motion",
					  G_OBJECT_CLASS_TYPE (widget_class),
					  G_SIGNAL_RUN_LAST,
					  G_STRUCT_OFFSET (SimpleCanvasClass, simple_canvas_pointer_motion),
					  NULL, NULL,
					  g_cclosure_marshal_VOID__POINTER,
					  G_TYPE_NONE,
					  1,
					  G_TYPE_POINTER );

	signals[SCANVAS_IMAGE_CHANGED] =
		g_signal_new ("simple-canvas-image-changed",
					   G_OBJECT_CLASS_TYPE (widget_class),
					  G_SIGNAL_RUN_LAST,
					  G_STRUCT_OFFSET (SimpleCanvasClass, simple_canvas_image_changed),
					  NULL, NULL,
					  g_cclosure_marshal_VOID__VOID,
					  G_TYPE_NONE, 0);
}

static void
simple_canvas_unrealize (GtkWidget *widget)
{
	SimpleCanvas *canvas;
	SimpleCanvasPrivate *priv;

	canvas = SIMPLE_CANVAS (widget);
	priv = simple_canvas_get_instance_priv(canvas);

	if(priv->surface){
	  cairo_surface_destroy (priv->surface);
	}
	if(priv->surf_tmp){
	  cairo_surface_destroy (priv->surf_tmp);
	}
	if(priv->background){
	  cairo_surface_destroy(priv->background);
	}

	GTK_WIDGET_CLASS (simple_canvas_parent_class)->unrealize (widget);
}

static void
simple_canvas_init (SimpleCanvas *canvas)
{
	SimpleCanvasPrivate *priv = simple_canvas_get_instance_priv(canvas);

	gtk_widget_set_can_focus(GTK_WIDGET(canvas), TRUE);
	gtk_widget_set_receives_default(GTK_WIDGET(canvas), TRUE);

	priv->line_width = 2;

	priv->aa = CAIRO_ANTIALIAS_DEFAULT;
	priv->fg.r = priv->fg.g = priv->fg.b = 0;
	priv->fg.a = 1.;
	priv->bg.r = priv->bg.g = priv->bg.b = 1.;
	priv->bg.a = 1;

	gtk_widget_add_events(GTK_WIDGET(canvas), GDK_ALL_EVENTS_MASK);
}


GtkWidget*
simple_canvas_new (int width, int height)
{
	SimpleCanvas *canvas;
	SimpleCanvasPrivate *priv;
	cairo_t *cr;

	canvas = g_object_new (SIMPLE_TYPE_CANVAS, NULL);
	if(SIMPLE_IS_CANVAS(canvas)){
		priv = simple_canvas_get_instance_priv(canvas);

		priv->surface = cairo_image_surface_create(DEF_SURF_FORMAT, width, height);
		priv->surf_tmp = cairo_image_surface_create(DEF_SURF_FORMAT, width, height);

		cr = cairo_create(priv->surface);
		cairo_set_source_rgba(cr, 0., 0., 0., 0.);
		cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
		cairo_paint (cr);
		cairo_destroy(cr);

		cr = cairo_create(priv->surf_tmp);
		cairo_set_source_surface (cr, priv->surface, 0., 0.);
		cairo_paint (cr);
		cairo_destroy(cr);

		gtk_widget_set_size_request(GTK_WIDGET(canvas), width, height);
		priv->zoom = 1.;
		priv->show_grid = TRUE;
		priv->max_zoom = MAX_ZOOM;
		priv->checkerb_bg = TRUE;
		priv->background = simple_canvas_create_background();
	}
	return GTK_WIDGET(canvas);
}

double win_to_img(double n, double z)
{
	return floor(n / z);
}

double img_to_win(double n, double z)
{
	return floor(n * z);
}

static void init_cr(SimpleCanvasPrivate *priv, cairo_t *cr)
{
	cairo_set_line_width(cr, (double)priv->line_width);
	cairo_set_source_rgba(cr, priv->fg.r, priv->fg.g, priv->fg.b, priv->fg.a);
	cairo_set_antialias(cr, priv->aa);
}

/*********************************************************************/

static gboolean
cairo_get_clip_rectangle(cairo_t *cr, GdkRectangle *rect)
{
	double x1, y1, x2, y2;
	gboolean clip_exists;

	cairo_clip_extents (cr, &x1, &y1, &x2, &y2);

	clip_exists = x1 < x2 && y1 < y2;

	if(rect){
		x1 = floor (x1);
		y1 = floor (y1);
		x2 = ceil (x2);
		y2 = ceil (y2);

		rect->x      = CLAMP (x1,      G_MININT, G_MAXINT);
		rect->y      = CLAMP (y1,      G_MININT, G_MAXINT);
		rect->width  = CLAMP (x2 - x1, G_MININT, G_MAXINT);
		rect->height = CLAMP (y2 - y1, G_MININT, G_MAXINT);
	}

	return clip_exists;
}

static void draw_background(GtkWidget *canvas, cairo_t *cr, GtkAllocation a)
{
	SimpleCanvasPrivate *priv = simple_canvas_get_instance_priv(SIMPLE_CANVAS (canvas));
	cairo_surface_t *surface;

	surface = priv->background;

	cairo_set_source_surface (cr, surface, 0., 0.);
	cairo_pattern_set_extend (cairo_get_source (cr), CAIRO_EXTEND_REPEAT);
	cairo_rectangle (cr, 0., 0., a.width, a.height);
	cairo_fill (cr);
}

static void draw_to_tmp(GtkWidget *canvas, GdkRectangle r)
{
	SimpleCanvasPrivate *priv = simple_canvas_get_instance_priv(SIMPLE_CANVAS (canvas));
	cairo_t *cr = cairo_create(priv->surf_tmp);
	double zoom = priv->zoom;

	cairo_set_source_surface(cr, priv->surface, 0., 0.);

	cairo_rectangle(cr,
					win_to_img((double)r.x, zoom)       - 2,
					win_to_img((double)r.y, zoom)       - 2,
					win_to_img((double)r.width, zoom)   + 4,
					win_to_img((double)r.height, zoom)  + 4);
	cairo_clip(cr);
	/* Erase previous contents */
	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
	cairo_paint(cr);

	cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
	init_cr(priv, cr);

	g_signal_emit (canvas, signals[SCANVAS_EXPOSE], 0, cr);

	cairo_destroy(cr);
}

static gboolean canvas_draw(GtkWidget *canvas, cairo_t *cr)
{
	SimpleCanvasPrivate *priv = simple_canvas_get_instance_priv(SIMPLE_CANVAS (canvas));
	GdkRectangle clip_rect;
	double zoom = priv->zoom;

	cairo_save(cr);

	if(!priv->checkerb_bg){
		cairo_set_source_rgba(cr, 1., 1., 1., 1.);
		cairo_paint(cr);
	}
	else{
		GtkAllocation a;

		gtk_widget_get_allocation(canvas, &a);
		draw_background(canvas, cr, a);
	}

	cairo_get_clip_rectangle(cr, &clip_rect);
	draw_to_tmp(canvas, clip_rect);

	cairo_scale(cr, zoom, zoom);
	cairo_set_source_surface(cr, priv->surf_tmp, 0., 0.);
	cairo_pattern_set_filter(cairo_get_source(cr), CAIRO_FILTER_FAST);
	cairo_paint(cr);

	cairo_restore(cr);

	if(zoom >= 8. && priv->show_grid){
		double x, y, w, h;
		GtkAllocation allocation;

		gtk_widget_get_allocation(canvas, &allocation);

		cairo_set_line_width(cr, 1.);
		cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
		cairo_set_source_rgb(cr, 0.75, 0.75, 0.75);

		w = (double)allocation.width + zoom;
		h = (double)allocation.height + zoom;

		for(x = 0.; x < w; x += zoom){
			cairo_move_to(cr, x, 0.);
			cairo_line_to(cr, x, h);
			cairo_stroke(cr);
		}

		for(y = 0.; y < h; y += zoom){
			cairo_move_to(cr, 0., y);
			cairo_line_to(cr, w, y);
			cairo_stroke(cr);
		}
	}
#if 0
{
	GdkRectangle area = clip_rect;

	cairo_set_line_width(cr, 1.);
	cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
	cairo_set_source_rgb(cr, 1., 0., 0.);
	area.x += 1;
	area.y += 1;
	area.width -= 2;
	area.height -= 2;
	gdk_cairo_rectangle(cr, &area);
	cairo_stroke(cr);
}
#endif
	return TRUE;
}
#if GTK_MAJOR_VERSION < 3
static
gint canvas_expose(GtkWidget *canvas, GdkEventExpose *event)
{
	if(gtk_widget_get_visible(canvas) && gtk_widget_get_mapped(canvas)){
		cairo_t *cr;
		GdkWindow *window = gtk_widget_get_window(canvas);

		cr = gdk_cairo_create(window);

		gdk_cairo_rectangle(cr, &(event->area));
		cairo_clip(cr);

		canvas_draw(canvas, cr);

		cairo_destroy(cr);

	}
	return FALSE;
}
#endif
/*********************************************************************/

static gint canvas_button_press(GtkWidget *widget, GdkEventButton *event)
{
	SimpleCanvasPrivate *priv = simple_canvas_get_instance_priv(SIMPLE_CANVAS (widget));

	GdkEventButton *copy = (GdkEventButton *)gdk_event_copy((GdkEvent *)event);

	if(event->type == GDK_BUTTON_PRESS){
		if(!gtk_widget_has_focus(widget))
			gtk_widget_grab_focus(widget);
	}

	copy->x = win_to_img(copy->x, priv->zoom);
	copy->y = win_to_img(copy->y, priv->zoom);
	priv->press.x = (int)copy->x;
	priv->press.y = (int)copy->y;

	g_signal_emit (widget, signals[SCANVAS_BUTTON_PRESS], 0, copy);
	gdk_event_free((GdkEvent *)copy);

	return TRUE;
}

static gint canvas_button_release(GtkWidget *widget, GdkEventButton *event)
{
	SimpleCanvasPrivate *priv = simple_canvas_get_instance_priv(SIMPLE_CANVAS (widget));

	GdkEventButton *copy = (GdkEventButton *)gdk_event_copy((GdkEvent *)event);

	copy->x = win_to_img(copy->x, priv->zoom);
	copy->y = win_to_img(copy->y, priv->zoom);
	priv->release.x = (int)copy->x;
	priv->release.y = (int)copy->y;

	g_signal_emit (widget, signals[SCANVAS_BUTTON_RELEASE], 0, copy);
	gdk_event_free((GdkEvent *)copy);

	return  TRUE;
}

static gint canvas_motion(GtkWidget *widget, GdkEventMotion *event)
{
	GdkEventMotion *copy = (GdkEventMotion *)gdk_event_copy((GdkEvent *)event);
	SimpleCanvasPrivate *priv = simple_canvas_get_instance_priv(SIMPLE_CANVAS (widget));

	copy->x = win_to_img(copy->x, priv->zoom);
	copy->y = win_to_img(copy->y, priv->zoom);
	g_signal_emit (widget, signals[SCANVAS_POINTER_MOTION], 0, copy);
	gdk_event_free((GdkEvent *)copy);

	return  TRUE;
}

cairo_t *simple_canvas_get_context(SimpleCanvas *canvas)
{
	SimpleCanvasPrivate *priv;
	cairo_t *cr;

	g_return_val_if_fail(SIMPLE_IS_CANVAS(canvas), NULL);

	priv = simple_canvas_get_instance_priv(canvas);

	cr = cairo_create(priv->surface);

	init_cr(priv, cr);

	return cr;
}

cairo_t *simple_canvas_get_window_context(SimpleCanvas *canvas)
{
	SimpleCanvasPrivate *priv;
	GdkWindow *window;
	cairo_t *cr;

	g_return_val_if_fail(SIMPLE_IS_CANVAS(canvas), NULL);

	window = gtk_widget_get_window(GTK_WIDGET(canvas));
	g_return_val_if_fail(GDK_IS_WINDOW(window), NULL);
	priv = simple_canvas_get_instance_priv(canvas);

#if GTK_CHECK_VERSION(3, 22, 0)
	G_GNUC_BEGIN_IGNORE_DEPRECATIONS
#endif

	cr = gdk_cairo_create(window);

#if GTK_CHECK_VERSION(3, 22, 0)
	G_GNUC_END_IGNORE_DEPRECATIONS
#endif

	init_cr(priv, cr);

	return cr;
}

void simple_canvas_get_release_pt(SimpleCanvas *canvas, GdkPoint *pt)
{
	if(SIMPLE_IS_CANVAS(canvas) && NULL != pt){
		SimpleCanvasPrivate *priv;

		priv = simple_canvas_get_instance_priv(canvas);
		*pt = priv->release;
	}
}

void simple_canvas_get_press_pt(SimpleCanvas *canvas, GdkPoint *pt)
{
	if(SIMPLE_IS_CANVAS(canvas) && NULL != pt){
		SimpleCanvasPrivate *priv;

		priv = simple_canvas_get_instance_priv(canvas);
		*pt = priv->press;
	}
}

void simple_canvas_set_image(SimpleCanvas *canvas, cairo_surface_t *surface)
{
	SimpleCanvasPrivate *priv;
	int w, h;
	cairo_status_t surface_status;

	g_return_if_fail(SIMPLE_IS_CANVAS(canvas));
	g_return_if_fail(surface != NULL);

	surface_status = cairo_surface_status(surface);
	g_return_if_fail(CAIRO_STATUS_SUCCESS == surface_status);

	w = cairo_image_surface_get_width(surface);
	h = cairo_image_surface_get_height(surface);
	g_return_if_fail(w > 0 && h > 0);

	priv = simple_canvas_get_instance_priv(canvas);
	if(priv->surface){
		cairo_surface_destroy(priv->surface);
	}
	priv->surface = surface;

	w = cairo_image_surface_get_width(priv->surface);
	h = cairo_image_surface_get_height(priv->surface);

	if(cairo_image_surface_get_width(priv->surf_tmp) != w ||
	   cairo_image_surface_get_height(priv->surf_tmp) != h){
	
		cairo_surface_t *tmp = cairo_image_surface_create(DEF_SURF_FORMAT, w, h);
		if(tmp){
			cairo_t *cr;

			cairo_surface_destroy(priv->surf_tmp);
			priv->surf_tmp = tmp;
			cr = cairo_create(priv->surf_tmp);
			cairo_set_source_surface (cr, priv->surface, 0, 0);
			cairo_paint (cr);

			cairo_destroy(cr);
		}
	}

	gtk_widget_set_size_request(GTK_WIDGET(canvas),
								(int)(img_to_win((double)w, priv->zoom)),
								(int)(img_to_win((double)h, priv->zoom)));

	gtk_widget_queue_draw(GTK_WIDGET(canvas));
	g_signal_emit (canvas, signals[SCANVAS_IMAGE_CHANGED], 0);
}

cairo_surface_t *simple_canvas_swap_image(SimpleCanvas *canvas, cairo_surface_t *surface)
{
	SimpleCanvasPrivate *priv;
	cairo_surface_t *old = NULL;
	int w, h;
	cairo_status_t surface_status;

	g_return_val_if_fail(SIMPLE_IS_CANVAS(canvas), NULL);
	g_return_val_if_fail(surface != NULL, NULL);

	surface_status = cairo_surface_status(surface);
	g_return_val_if_fail(CAIRO_STATUS_SUCCESS == surface_status, NULL);

	priv = simple_canvas_get_instance_priv(canvas);
	old = priv->surface;
	priv->surface = surface;

	w = cairo_image_surface_get_width(priv->surface);
	h = cairo_image_surface_get_height(priv->surface);

	if(cairo_image_surface_get_width(priv->surf_tmp) != w ||
	   cairo_image_surface_get_height(priv->surf_tmp) != h){
	
		cairo_surface_t *tmp = cairo_image_surface_create(DEF_SURF_FORMAT, w, h);
		if(tmp){
			cairo_t *cr;
			cairo_surface_destroy(priv->surf_tmp);
			priv->surf_tmp = tmp;
			cr = cairo_create(priv->surf_tmp);
			cairo_set_source_surface (cr, priv->surface, 0, 0);
			cairo_paint (cr);

			cairo_destroy(cr);
		}
	}

	gtk_widget_set_size_request(GTK_WIDGET(canvas),
								(int)(img_to_win((double)cairo_image_surface_get_width(priv->surface), priv->zoom)),
								(int)(img_to_win((double)cairo_image_surface_get_height(priv->surface), priv->zoom)));
	gtk_widget_queue_draw(GTK_WIDGET(canvas));

	g_signal_emit (canvas, signals[SCANVAS_IMAGE_CHANGED], 0);

	return old;
}

cairo_surface_t *simple_canvas_get_image(SimpleCanvas *canvas)
{
	SimpleCanvasPrivate *priv;
	g_return_val_if_fail(SIMPLE_IS_CANVAS(canvas), NULL);

	priv = simple_canvas_get_instance_priv(canvas);

	return priv->surface;
}

double simple_canvas_set_zoom(SimpleCanvas *canvas, double zoom)
{
	SimpleCanvasPrivate *priv;
	double old = 0;
	g_return_val_if_fail(SIMPLE_IS_CANVAS(canvas), 0);

	priv = simple_canvas_get_instance_priv(canvas);
	old = priv->zoom;
	if(zoom >= .1 && zoom <= priv->max_zoom){
		priv->zoom = zoom;

		gtk_widget_set_size_request(GTK_WIDGET(canvas),
								(int)(img_to_win((double)cairo_image_surface_get_width(priv->surface), priv->zoom)),
								(int)(img_to_win((double)cairo_image_surface_get_height(priv->surface), priv->zoom)));
		gtk_widget_queue_resize(GTK_WIDGET(canvas));
	}

	return old;
}

double simple_canvas_get_zoom(SimpleCanvas *canvas)
{
	SimpleCanvasPrivate *priv;
	g_return_val_if_fail(SIMPLE_IS_CANVAS(canvas), 0);
	priv = simple_canvas_get_instance_priv(canvas);

	return priv->zoom;
}

void simple_canvas_set_max_zoom(SimpleCanvas *canvas, double zoom)
{
	SimpleCanvasPrivate *priv;
	g_return_if_fail(SIMPLE_IS_CANVAS(canvas));

	priv = simple_canvas_get_instance_priv(canvas);

	if(zoom >= .1 && zoom <= MAX_ZOOM){
		priv->max_zoom = zoom;
	}
}

void simple_canvas_set_show_grid(SimpleCanvas *canvas, gboolean show_grid)
{
	SimpleCanvasPrivate *priv;

	g_return_if_fail(SIMPLE_IS_CANVAS(canvas));

	priv = simple_canvas_get_instance_priv(canvas);
	priv->show_grid = show_grid;
	gtk_widget_queue_draw(GTK_WIDGET(canvas));
}

void simple_canvas_update_rect(SimpleCanvas *sc, GdkRectangle *r)
{
	SimpleCanvasPrivate *priv;
	g_return_if_fail(SIMPLE_IS_CANVAS(sc));
	priv = simple_canvas_get_instance_priv(sc);

	if(r){
		GdkWindow *window = gtk_widget_get_window(GTK_WIDGET(sc));

		r->x = (int)img_to_win((double)r->x, priv->zoom);
		r->y = (int)img_to_win((double)r->y, priv->zoom);
		r->width = (int)img_to_win((double)r->width, priv->zoom);
		r->height = (int)img_to_win((double)r->height, priv->zoom);

		gdk_window_invalidate_rect(window, r, 0);
	}
}

int simple_canvas_get_line_width(SimpleCanvas *sc)
{
	if(SIMPLE_IS_CANVAS(sc)){
		SimpleCanvasPrivate *priv;
		priv = simple_canvas_get_instance_priv(sc);
		return priv->line_width;
	}
	return 0;
}

void simple_canvas_set_line_width(SimpleCanvas *sc, int line_width)
{
	if(SIMPLE_IS_CANVAS(sc) && line_width > 0){
		SimpleCanvasPrivate *priv = simple_canvas_get_instance_priv(sc);
		priv->line_width = line_width;
	}
}

void simple_canvas_get_fg_rgba(SimpleCanvas *sc, double *r, double *g, double *b, double *a)
{
	SimpleCanvasPrivate *priv;
	g_return_if_fail(SIMPLE_IS_CANVAS(sc));
	g_return_if_fail(r && g && b && a);
	priv = simple_canvas_get_instance_priv(sc);

	*r = priv->fg.r;
	*g = priv->fg.g;
	*b = priv->fg.b;
	*a = priv->fg.a;
}

void simple_canvas_get_bg_rgba(SimpleCanvas *sc, double *r, double *g, double *b, double *a)
{
	SimpleCanvasPrivate *priv;
	g_return_if_fail(SIMPLE_IS_CANVAS(sc));
	g_return_if_fail(r && g && b && a);
	priv = simple_canvas_get_instance_priv(sc);

	*r = priv->bg.r;
	*g = priv->bg.g;
	*b = priv->bg.b;
	*a = priv->bg.a;
}

void simple_canvas_set_fg_rgba(SimpleCanvas *sc, double r, double g, double b, double a)
{
	SimpleCanvasPrivate *priv;
	g_return_if_fail(SIMPLE_IS_CANVAS(sc));
	priv = simple_canvas_get_instance_priv(sc);

	priv->fg.r = r;
	priv->fg.g = g;
	priv->fg.b = b;
	priv->fg.a = a;
}

void simple_canvas_set_bg_rgba(SimpleCanvas *sc, double r, double g, double b, double a)
{
	SimpleCanvasPrivate *priv;
	g_return_if_fail(SIMPLE_IS_CANVAS(sc));
	priv = simple_canvas_get_instance_priv(sc);

	priv->bg.r = r;
	priv->bg.g = g;
	priv->bg.b = b;
	priv->bg.a = a;
}

cairo_antialias_t simple_canvas_get_antialias(SimpleCanvas *sc)
{
	if(SIMPLE_IS_CANVAS(sc)){
		SimpleCanvasPrivate *priv;
		priv = simple_canvas_get_instance_priv(sc);
		return (priv->aa);
	}

	return CAIRO_ANTIALIAS_DEFAULT;
}

void simple_canvas_set_antialias(SimpleCanvas *sc, cairo_antialias_t aa)
{
	if(SIMPLE_IS_CANVAS(sc)){
		SimpleCanvasPrivate *priv;
		priv = simple_canvas_get_instance_priv(sc);
		priv->aa = aa;
	}
}

void simple_canvas_set_use_checkerboard_bg(SimpleCanvas *sc, gboolean check_bg)
{
	if(SIMPLE_IS_CANVAS(sc)){
		SimpleCanvasPrivate *priv;

		priv = simple_canvas_get_instance_priv(sc);
		priv->checkerb_bg = check_bg;

		gtk_widget_queue_draw(GTK_WIDGET(sc));
	}
}
