/* 
 * (c) 2014 - 2023 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "util.h"
#include "dibuja.h"
#include "color.h"
#include "simple-canvas.h"
#include "file.h"


#define save_widget(wn) (save_widget_as((builder),(window), \
						 (wn),(wn)))

typedef enum{
	TP_LINE		= 0,
	TP_BRUSH	= 1,
	TP_SHAPE	= 2,
	TP_PENCIL	= 3,
	TP_SELECT	= 4,
	TP_FFILL	= 5
}ToolPage;

void msg_dlg_forall(GtkWidget *widget, gpointer data);
void dbj_dialog_add_button(GtkDialog *dlg, const char *name, const char *text, gint id, gboolean def_resp);
void util_set_tool_page(int tooltype, int page_num)
{
	gtk_notebook_set_current_page(GTK_NOTEBOOK(get_widget("Tool Properties")), page_num);

	switch(page_num){
		case TP_SHAPE://2:
		case TP_LINE://0:
			if(0 == tooltype){
				gtk_widget_show(get_widget("check_button_use_arrow"));
			}
			else{
				gtk_widget_hide(get_widget("check_button_use_arrow"));
			}
			gtk_widget_show(get_widget("check_button_antialias"));
			gtk_widget_show(get_widget("label_line_width"));
			gtk_widget_show(get_widget("spinbutton_line_width"));
			break;
		case TP_BRUSH://1:
		case TP_PENCIL://3:
			gtk_widget_show(get_widget("check_button_antialias"));
			gtk_widget_hide(get_widget("label_line_width"));
			gtk_widget_hide(get_widget("spinbutton_line_width"));
			break;
		case TP_SELECT://4:
			gtk_widget_hide(get_widget("check_button_antialias"));
			gtk_widget_hide(get_widget("label_line_width"));
			gtk_widget_hide(get_widget("spinbutton_line_width"));
			break;
		case TP_FFILL://5:
			gtk_widget_hide(get_widget("check_button_antialias"));
			gtk_widget_hide(get_widget("label_line_width"));
			gtk_widget_hide(get_widget("spinbutton_line_width"));
			break;
		default:
			gtk_widget_show(get_widget("check_button_antialias"));
			gtk_widget_show(get_widget("label_line_width"));
			gtk_widget_show(get_widget("spinbutton_line_width"));
			break;
	}
}

gchar *text_view_get_text(GtkTextView *textview, gboolean hidden_chars)
{
	GtkTextIter start, end;
	GtkTextBuffer *buffer;
	gchar *text = NULL;

	g_return_val_if_fail(GTK_IS_TEXT_VIEW(textview), NULL);

	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
	gtk_text_buffer_get_start_iter(buffer, &start);
	gtk_text_buffer_get_end_iter(buffer, &end);
	text = gtk_text_buffer_get_text(buffer, &start, &end, hidden_chars);

	return text;
}

void message_box_ok(GtkMessageType type, gchar const *format, ...)
{
	gchar *buffer;
	va_list args;
	GtkWidget *dialog, *action_area;

	va_start (args, format);
	buffer = g_strdup_vprintf (format, args);
	va_end (args);

	dialog = gtk_message_dialog_new (get_window(),
                                 GTK_DIALOG_DESTROY_WITH_PARENT, type,
                                 GTK_BUTTONS_NONE,
                                 "%s", buffer);

	dbj_dialog_add_button(GTK_DIALOG(dialog), STOCK_CLOSE, _("_Close"),
						  GTK_RESPONSE_YES, TRUE);
	gtk_window_set_title(GTK_WINDOW(dialog), PACKAGE_NAME);
G_GNUC_BEGIN_IGNORE_DEPRECATIONS
	action_area = gtk_dialog_get_action_area(GTK_DIALOG (dialog));
G_GNUC_END_IGNORE_DEPRECATIONS
	gtk_button_box_set_layout(GTK_BUTTON_BOX(action_area), GTK_BUTTONBOX_CENTER);
#if GTK_CHECK_VERSION(3, 12, 0)
	GtkWidget *image;
	switch(type){
		case GTK_MESSAGE_INFO:
			image = gtk_image_new_from_icon_name("dialog-info", GTK_ICON_SIZE_DIALOG);
			break;
 
		case GTK_MESSAGE_QUESTION:
			image = gtk_image_new_from_icon_name("dialog-question", GTK_ICON_SIZE_DIALOG);
			break;
 
		case GTK_MESSAGE_WARNING:
			image = gtk_image_new_from_icon_name("dialog-warning", GTK_ICON_SIZE_DIALOG);
			break;
 
		case GTK_MESSAGE_ERROR: // Fall through
		default:
			image = gtk_image_new_from_icon_name("dialog-error", GTK_ICON_SIZE_DIALOG);
			break;
	}
	g_object_set(image, "visible", TRUE, NULL);
	gtk_widget_set_size_request(image, 48, 48);
	g_object_set(dialog, "image", image, NULL);
#endif
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
	g_free(buffer);
}

gint message_box_yes_no(gchar const *format, ...)
{
	gchar *buffer;
	va_list args;
	GtkWidget *dialog;
	gint ret;

	va_start (args, format);
	buffer = g_strdup_vprintf (format, args);
	va_end (args);

	dialog = gtk_message_dialog_new (get_window(),
                                 GTK_DIALOG_DESTROY_WITH_PARENT,
                                 GTK_MESSAGE_QUESTION,
                                 GTK_BUTTONS_NONE,
                                 "%s", buffer);

	dbj_dialog_add_button(GTK_DIALOG(dialog), STOCK_YES, _("_Yes"),
						  GTK_RESPONSE_YES, TRUE);
	dbj_dialog_add_button(GTK_DIALOG(dialog), STOCK_NO, _("_No"),
						  GTK_RESPONSE_NO, FALSE);
	gtk_window_set_title(GTK_WINDOW(dialog), PACKAGE_NAME);
#if  GTK_CHECK_VERSION(3, 12, 0)
	GtkWidget *image = gtk_image_new_from_icon_name("dialog-question", GTK_ICON_SIZE_DIALOG);
	g_object_set(image, "visible", TRUE, NULL);
	g_object_set(dialog, "image", image, NULL);
#endif
	ret = gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
	g_free(buffer);

	return ret;
}
// Takes care of ugly selected text in dlg
void msg_dlg_forall(GtkWidget *widget, gpointer data)
{
	char *name = NULL;

	name = (char *)gtk_widget_get_name(widget);

	if(0 == g_strcmp0(name, "GtkHBox")){
		gtk_container_forall(GTK_CONTAINER(widget), msg_dlg_forall, NULL);
	}
	else if(0 == g_strcmp0(name, "GtkVBox")){
		gtk_container_forall(GTK_CONTAINER(widget), msg_dlg_forall, NULL);
	}
	else if(0 == g_strcmp0(name, "GtkLabel")){
		gtk_widget_set_can_focus(widget, FALSE);
		gtk_widget_queue_draw(widget);
	}
}

void message_box_error(gchar const *format, ...)
{
	gchar *buffer;
	va_list args;
	GtkWidget *dialog, *vbox;

	va_start (args, format);
	buffer = g_strdup_vprintf (format, args);
	va_end (args);

	dialog = gtk_message_dialog_new (get_window(),
                                 GTK_DIALOG_DESTROY_WITH_PARENT,
                                 GTK_MESSAGE_ERROR,
                                 GTK_BUTTONS_NONE, "%s", buffer);

	dbj_dialog_add_button(GTK_DIALOG(dialog), STOCK_CLOSE, _("_Close"),
						  GTK_RESPONSE_CLOSE, TRUE);
	vbox = gtk_dialog_get_content_area(GTK_DIALOG (dialog));
	gtk_container_forall(GTK_CONTAINER(vbox), msg_dlg_forall, NULL);
	gtk_window_set_title(GTK_WINDOW(dialog), PACKAGE_NAME);
#if  GTK_CHECK_VERSION(3, 12, 0)
	GtkWidget *image = gtk_image_new_from_icon_name("dialog-error", GTK_ICON_SIZE_DIALOG);
	g_object_set(image, "visible", TRUE, NULL);
	g_object_set(dialog, "image", image, NULL);
#endif
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
	g_free(buffer);
}
// "Close without saving", "cancel", "save"
gint message_box_yes_no_cancel(gchar const *format, ...)
{
	gchar *buffer;
	va_list args;
	GtkWidget *dialog;
	gint ret;

	va_start (args, format);
	buffer = g_strdup_vprintf (format, args);
	va_end (args);

	dialog = gtk_message_dialog_new (get_window(),
                                 GTK_DIALOG_DESTROY_WITH_PARENT,
                                 GTK_MESSAGE_QUESTION,

                                 GTK_BUTTONS_NONE, "%s", buffer);

	dbj_dialog_add_button(GTK_DIALOG(dialog), STOCK_YES, _("_Yes"),
						  GTK_RESPONSE_YES, TRUE);
	dbj_dialog_add_button(GTK_DIALOG(dialog), STOCK_NO, _("_No"),
						  GTK_RESPONSE_NO, FALSE);
	dbj_dialog_add_button(GTK_DIALOG(dialog), STOCK_CANCEL, _("_Cancel"),
						  GTK_RESPONSE_CANCEL, FALSE);

	gtk_window_set_title(GTK_WINDOW(dialog), PACKAGE_NAME);
#if  GTK_CHECK_VERSION(3, 12, 0)
	GtkWidget *image = gtk_image_new_from_icon_name("dialog-question", GTK_ICON_SIZE_DIALOG);
	g_object_set(image, "visible", TRUE, NULL);
	g_object_set(dialog, "image", image, NULL);
#endif
	ret = gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
	g_free(buffer);

	return ret;
}
/*
void rect_from_pts(int p1x, int p1y, int p2x, int p2y, GdkRectangle *rectangle)
{
	if(!rectangle) return;

	rectangle->x = MIN(p1x, p2x);
	rectangle->y = MIN(p1y, p2y);
	rectangle->width = ABS(p1x - p2x);
	rectangle->height = ABS(p1y - p2y);
}

void resize_rect(GdkRectangle *rectangle, int x, int y)
{
	if(!rectangle) return;

	rectangle->x -= x;
	rectangle->y -= y;
	rectangle->width += y * 2;
	rectangle->height += y * 2;
}
*/
/*void set_update_rect_orig(GdkRectangle *r, int p1x, int p1y, int p2x, int p2y, double z)
{

	if(r){
		r->x = (int)img_to_win((double)(MIN(p1x, p2x) - 2), z);
		r->y = (int)img_to_win((double)(MIN(p1y, p2y) - 2), z);
		r->width = (int)img_to_win((double)(ABS(p1x - p2x) + 4), z);
		r->height = (int)img_to_win((double)(ABS(p1y - p2y) + 4), z);
		resize_rect(r, 4, 4);
	}
}*/

void set_update_rect(GdkRectangle *r, int p1x, int p1y, int p2x, int p2y, double z)
{

	if(r){
		r->x = (int)img_to_win((double)(MIN(p1x, p2x)), z);
		r->y = (int)img_to_win((double)(MIN(p1y, p2y)), z);
		r->width = (int)img_to_win((double)(ABS(p1x - p2x)), z);
		r->height = (int)img_to_win((double)(ABS(p1y - p2y)), z);
		//resize_rect(r, 4, 4);
	}
}
/*
void debug_rect(CR_TYPE type, GdkRectangle *r)
{
	cairo_t *cr;
	
	if(!r) return;
	cr = dibuja_get_cr(type);
	cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
	cairo_set_line_width(cr, 1.0);
	gdk_cairo_rectangle(cr, r);
	cairo_stroke(cr);
	cairo_destroy(cr);
}*/
/*
void debug_rect_from_pts(CR_TYPE type, int p1x, int p1y, int p2x, int p2y)
{
	GdkRectangle r;

	rect_from_pts(p1x, p1y, p2x, p2y, &r);
	debug_rect(type, &r);
}
*/
void canvas_set_wait_cursor(GtkWidget *widget)
{
	GdkCursor *cur;

	if(!GDK_IS_WINDOW (gtk_widget_get_window(widget))) return;

	cur = gdk_cursor_new_for_display(gdk_display_get_default(), GDK_WATCH);
	if(cur)
		gdk_window_set_cursor(gtk_widget_get_window(widget), cur);
}

GdkCursor *canvas_load_cursor(GdkCursorType type)
{
	GdkPixbuf *pixbuf = NULL;
	GdkCursor *cur = NULL;

	pixbuf = file_load_pixbuf(DIBUJA_CURSOR_DIR	"dibuja-crosshair-cursor.png", FALSE);

	if(!GDK_IS_PIXBUF(pixbuf))
		return gdk_cursor_new_for_display(gdk_display_get_default(), GDK_LEFT_PTR);

	cur = gdk_cursor_new_from_pixbuf(gdk_display_get_default(), pixbuf, 10, 10);
	return cur;
}

void canvas_set_cursor(GtkWidget *widget, GdkCursorType type)
{
	GdkCursor *cur;
	
	if(!GDK_IS_WINDOW (gtk_widget_get_window(widget))) return;
	if((DIBUJA_CURSOR != type) && !(type >= GDK_X_CURSOR && type < GDK_LAST_CURSOR)) return;

	if(DIBUJA_CURSOR == type){
		cur = canvas_load_cursor(type);
	}
	else{
		cur = gdk_cursor_new_for_display(gdk_display_get_default(), type);
	}

	if(cur)
		gdk_window_set_cursor(gtk_widget_get_window(widget), cur);
	else{
		g_message("Error creating cursor\n");
	}
}

/***************************************************************************/
void util_get_display_info(GtkWindow *window, gint *w, gint *h,
							   gint *wmm, gint *hmm)
{
	g_return_if_fail(GTK_IS_WINDOW(window));

	if(!gtk_widget_get_realized(GTK_WIDGET(window)))
		gtk_widget_realize(GTK_WIDGET(window));

#if GTK_CHECK_VERSION(3, 22, 0)
	GdkDisplay *d;
	GdkMonitor *m;
	GdkRectangle r = {0};

	d = gtk_widget_get_display(GTK_WIDGET(window));
	m = gdk_display_get_monitor_at_window(d, gtk_widget_get_window(GTK_WIDGET(window)));
	gdk_monitor_get_geometry(m, &r);

	if(w) *w = r.width;
	if(h) *h = r.height;
	if(wmm) *wmm = gdk_monitor_get_width_mm(m);
	if(hmm) *hmm = gdk_monitor_get_height_mm(m);
#else
	GdkScreen *screen = gtk_widget_get_screen(GTK_WIDGET(window));

	if(w) *w = gdk_screen_get_width(screen);
	if(h) *h = gdk_screen_get_height(screen);
	if(wmm) *wmm = gdk_screen_get_width_mm(screen);
	if(hmm) *hmm = gdk_screen_get_height_mm(screen);
#endif
}

/**
 * gimp_get_screen_resolution:
 * @screen: a #GdkScreen or %NULL
 * @xres: returns the horizontal screen resolution (in dpi)
 * @yres: returns the vertical screen resolution (in dpi)
 *
 * Retrieves the screen resolution from GDK. If @screen is %NULL, the
 * default screen is used.
 **/
void
gimp_get_screen_resolution(GtkWindow *window,
                            gdouble   *xres,
                            gdouble   *yres)
{
  gint    width, height;
  gint    width_mm, height_mm;
  gdouble x = 0.0;
  gdouble y = 0.0;

  g_return_if_fail (GTK_IS_WINDOW(window));
  g_return_if_fail (xres != NULL && yres != NULL);

  util_get_display_info(window, &width, &height, &width_mm, &height_mm);

  /*
   * From xdpyinfo.c:
   *
   * there are 2.54 centimeters to an inch; so there are 25.4 millimeters.
   *
   *     dpi = N pixels / (M millimeters / (25.4 millimeters / 1 inch))
   *         = N pixels / (M inch / 25.4)
   *         = N * 25.4 pixels / M inch
   */

  if (width_mm > 0 && height_mm > 0)
    {
      x = (width  * 25.4) / (gdouble) width_mm;
      y = (height * 25.4) / (gdouble) height_mm;
    }

  if (x < GIMP_MIN_RESOLUTION || x > GIMP_MAX_RESOLUTION ||
      y < GIMP_MIN_RESOLUTION || y > GIMP_MAX_RESOLUTION)
    {
      g_warning ("GDK returned bogus values for the screen resolution, "
                 "using 96 dpi instead.");

      x = 96.0;
      y = 96.0;
    }

  /*  round the value to full integers to give more pleasant results  */
  *xres = ROUND (x);
  *yres = ROUND (y);
}

void util_spin_set_val(char *adj, gdouble value)
{
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget(adj)), value);
}
/***************************************************************************/
static void save_widget_as(GtkBuilder *builder, GtkWindow *window,
						   char *widget_name, char *save_as_name)
{
	GtkWidget *widget;

	widget = builder_get_widget(builder, widget_name);
	if(!GTK_IS_WIDGET(widget)) {
		g_warning("Could not save widget: \n"
				  "    \"%s\" as \"%s\"", widget_name, save_as_name);
		return;
	}
	g_object_set_data(G_OBJECT(window), save_as_name, (gpointer)widget);
}

static void util_save_menu_widgets(GtkWidget *win, GtkBuilder *builder)
{
	GtkWindow *window = GTK_WINDOW(win);

	// Edit Menu
	save_widget("menu2");
	save_widget_as(builder, window, "menu_edit", "editmenu");

	save_widget_as(builder, window, "menu_edit_undo", "edit_undo");
	gtk_widget_set_sensitive(get_widget("edit_undo"), FALSE);

	save_widget_as(builder, window, "menu_edit_redo", "edit_redo");
	gtk_widget_set_sensitive(get_widget("edit_redo"), FALSE);

	save_widget_as(builder, window, "menu_edit_paste", "edit_paste");
	save_widget_as(builder, window, "menu_edit_copy", "edit_copy");
	save_widget("menu_edit_cut");
	save_widget("menu_edit_clear");

// Image Menu
	save_widget("menu_image_crop");
	save_widget("menu_image_selection_as_brush");

// Effects menu
	save_widget("menu_effects_grayscale");
	save_widget("menu_effects_invert");
	save_widget("menu_effects_insta_brannan");
	save_widget("menu_effects_insta_1977");
	save_widget("menu_effects_pixelate");
	save_widget("menu_effects_brightness_contrast");
	save_widget("menu_effects_gaussian_blur");
	save_widget("menu_effects_posterize");
	save_widget("menu_effects_threshold");
	save_widget("menu_effects_sharpen");
	save_widget("menu_effects_vignette");
	save_widget("menu_effects_emboss");
	save_widget("menu_effects_pixelate");
	save_widget("menu_effects_saturation_hue");

// View menu
	save_widget_as(builder, window,
				   "menu_view_checkerboard_bg", "checkerb_bg");
}

static void util_save_toolbar_widgets(GtkWidget *win, GtkBuilder *builder)
{
	GtkWindow *window = GTK_WINDOW(win);

// Toolbar zoom combo & buttons
	save_widget("toolbar1");
	save_widget("tb_zoom_in");
	save_widget("tb_zoom_out");
	save_widget("tb_cut");

// Toolbar undo & redo buttons
	save_widget("tb_undo");
	gtk_widget_set_sensitive(get_widget("tb_undo"), FALSE);
	save_widget("tb_redo");
	gtk_widget_set_sensitive(get_widget("tb_redo"), FALSE);
}

void util_save_widgets(GtkWidget *win, GtkBuilder *builder)
{
	GtkWindow *window = GTK_WINDOW(win);

// Tools
	save_widget("check_button_antialias");
	save_widget("check_button_use_arrow");
	save_widget("label_line_width");
	save_widget("spinbutton_line_width");
	save_widget("table1");
	save_widget("spinbutton_brush_width");
	save_widget("spinbutton_brush_spacing");
	save_widget("Tool Properties");
	save_widget("tb_apply_selection");
	save_widget("cb_leave_cutout");

// Colors
	save_widget("table_col_pal");

// Statusbar
	save_widget("statusbar1");

// Text tool TODO: Nicer text tool
	save_widget_as(builder, window, "dialog-text", "text-dialog");
	save_widget_as(builder, window, "textview1", "textview");
	save_widget_as(builder, window, "check_box_use_fill_prop", "use_fill_prop");
	save_widget_as(builder, window, "check_button_center_horz", "center_horz");
	save_widget_as(builder, window, "check_button_center_vert", "center_vert");
	save_widget_as(builder, window, "check_button_meme_text", "meme-text");

// New File Dialog
	save_widget_as(builder, window, "dialog_new_file_at_size",
				   "dialog_new_image");
	save_widget_as(builder, window, "spin_new_image_width",
				   "new_image_width_spin");
	save_widget_as(builder, window, "spin_new_image_height",
				   "new_image_height_spin");
	save_widget_as(builder, window, "combo_filesize_type",
				   "filesize_type_combo");

// Rotate dialog
	save_widget("dialog_rotate");
	save_widget("dialog_rotate_darea");
	save_widget("dialog_rotate_angle_spin");

// Resize Dialog
	save_widget("dialog_resize");
	save_widget("dialog_resize_entry_width");
	save_widget("dialog_resize_entry_height");
	save_widget("dialog_resize_label_size");
	save_widget("dialog_resize_checkbutton");

// Scale Dialog
	save_widget("dialog_scale");
	save_widget("dialog_scale_label_size");
	save_widget("dialog_scale_spinbutton_width");
	save_widget("dialog_scale_spinbutton_height");
	save_widget("rb_dimensions");
	save_widget("rb_percentage");
	save_widget("scale_vbox2");
	save_widget("scale_vbox3");
	save_widget("label_percent_scale");

// Scroll
	save_widget_as(builder, window,
				   "scrolledwindow1", "scroll");

	util_save_menu_widgets(win, builder);
	util_save_toolbar_widgets(win, builder);
}

char *util_get_config_filename(void)
{
	char *filename;
	char *config_dir = util_get_dibuja_config_dir();

	filename = g_build_filename(config_dir, DIBUJA_CONFIG_FILE, NULL);
	g_free(config_dir);

	return filename;
}

char *util_get_dibuja_config_dir(void)
{
	char *filename;
#ifndef WORKING_DIR
	char *config_dir = (char *)g_get_user_config_dir();
	filename = g_build_filename(config_dir, PACKAGE, NULL);
#else
	filename = g_strdup(WORKING_DIR);
#endif

	return filename;
}

GtkWidget *util_create_label(const gchar *format, ...)
{
	GtkWidget *label;
	char *markup;
	va_list args;

	va_start(args, format);
	markup = g_markup_vprintf_escaped(format, args);
	va_end(args);

	label = gtk_label_new(NULL);
	if(markup){
		gtk_label_set_markup(GTK_LABEL(label), markup);
		g_free(markup);
	}

	return label;
}

void dbj_window_get_size(GdkWindow *window, gint *width, gint *height)
{
	g_return_if_fail(window && width && height);
#if GTK_CHECK_VERSION(2, 24, 0)
	*width = gdk_window_get_width(window);
	*height = gdk_window_get_height(window);
#else
	gdk_drawable_get_size(window, width, height);
#endif
}

/* Combo box =================================================================*/
GtkWidget *dib_combo_box_entry_new_text(void)
{
#if GTK_CHECK_VERSION(2, 24, 0)
	return gtk_combo_box_text_new_with_entry();
#else
	return gtk_combo_box_entry_new_text();
#endif
}

void dib_combo_box_append_text(GtkWidget *combo, gchar *text)
{
#if GTK_CHECK_VERSION(3, 0, 0)
	g_return_if_fail(GTK_IS_COMBO_BOX_TEXT(combo));
	gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(combo), NULL, text);
#elif GTK_CHECK_VERSION(2, 24, 0)
	g_return_if_fail(GTK_IS_COMBO_BOX_TEXT(combo));
	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo), text);
#else
	g_return_if_fail(GTK_IS_COMBO_BOX(combo));
	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), text);
#endif
}

char *dib_combo_box_get_active_text(GtkWidget *combo)
{
#if GTK_CHECK_VERSION(2, 24, 0)
	g_return_val_if_fail(GTK_IS_COMBO_BOX_TEXT(combo), NULL);
	return gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(combo));
#else
	g_return_val_if_fail(GTK_IS_COMBO_BOX(combo), NULL);
	return gtk_combo_box_get_active_text(GTK_COMBO_BOX(combo));
#endif
}

void dib_combo_box_set_active(GtkWidget *combo, gint index_)
{
	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), index_);
}

gint dib_combo_box_get_active(GtkWidget *combo)
{
	g_return_val_if_fail(GTK_IS_COMBO_BOX(combo), -1);
	return gtk_combo_box_get_active(GTK_COMBO_BOX(combo));
}

void dib_combo_box_insert_text(GtkWidget *combo, gint position, const gchar *text)
{
#if GTK_CHECK_VERSION(2, 24, 0)
	g_return_if_fail(GTK_IS_COMBO_BOX_TEXT(combo));
	gtk_combo_box_text_insert_text(GTK_COMBO_BOX_TEXT(combo),
								   position, text);
#else
	g_return_if_fail(GTK_IS_COMBO_BOX(combo));
	gtk_combo_box_insert_text(GTK_COMBO_BOX(combo),
							  position, text);
#endif
}
/*============================================================================*/

GtkBuilder *create_builder(const char *filename, GError **error)
{
	GtkBuilder *builder = NULL;

	builder = gtk_builder_new();
	if(gtk_builder_add_from_file(builder, filename, error))
		gtk_builder_connect_signals(builder, NULL);

	return builder;
}

GtkWidget *dib_table_new(guint rows, guint columns, gboolean homogeneous)
{
	gint *c, *r;
#if GTK_MAJOR_VERSION >= 3 && GTK_MINOR_VERSION >= 4
	GtkWidget *table = gtk_grid_new();
#else
	GtkWidget *table = gtk_table_new(rows, columns, homogeneous);
#endif
	c = g_malloc(sizeof(gint));
	*c = columns;
	r = g_malloc(sizeof(gint));
	*r = rows;

	g_object_set_data_full(G_OBJECT(table), "nrows", (gpointer)r, g_free);
	g_object_set_data_full(G_OBJECT(table), "ncolumns", (gpointer)c, g_free);

#if GTK_MAJOR_VERSION >= 3 && GTK_MINOR_VERSION >= 4
	gtk_grid_set_column_spacing(GTK_GRID(table), 2);
	gtk_grid_set_row_homogeneous(GTK_GRID(table), homogeneous);
	gtk_grid_set_column_homogeneous(GTK_GRID(table), homogeneous);

#endif

	return table;
}

void dib_table_attach(GtkWidget *table, GtkWidget *child,
					  guint left_attach, guint right_attach,
					  guint top_attach, guint bottom_attach,
					  DibAttachOptions xoptions, DibAttachOptions yoptions,
					  guint xpad, guint ypad)
{
#if GTK_MAJOR_VERSION >= 3 && GTK_MINOR_VERSION >= 4
	gtk_grid_attach(GTK_GRID(table), child, left_attach, top_attach,
					ABS(right_attach - left_attach),
					ABS(bottom_attach - top_attach));
	gtk_widget_set_hexpand(child, (xoptions & DIB_EXPAND));
//	gtk_widget_set_vexpand(child, (yoptions & DIB_EXPAND));
	g_object_set(child, "margin-start", xpad, "margin-end", xpad, NULL);
	g_object_set(child, "margin-top", xpad, "margin-bottom", ypad, NULL);
#else
	gtk_table_attach(GTK_TABLE(table), child, left_attach, right_attach,
					 top_attach, bottom_attach,
					 xoptions, yoptions, 1 + xpad, 1 + ypad);
#endif
}

void dib_table_attach_defaults(GtkWidget *table, GtkWidget *widget, 
							   guint lattach, guint rattach,
							   guint tattach, guint battach)
{
	dib_table_attach(table, widget, lattach, rattach,
					 tattach, battach, DIB_EXPAND | DIB_FILL,
					 DIB_EXPAND | DIB_FILL, 0, 0);

}

GtkWidget *image_from_default_theme(const char *name, GtkIconSize size)
{
	GtkIconTheme *theme;
	GdkPixbuf *pixbuf;
	GtkWidget *image = gtk_image_new();

	theme = gtk_icon_theme_get_default();
	pixbuf = gtk_icon_theme_load_icon(theme, name, size, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
	if(GDK_IS_PIXBUF(pixbuf)){
	    gtk_image_set_from_pixbuf(GTK_IMAGE(image), pixbuf);
	    g_object_unref(pixbuf);
	}

	return image;
}

void dbj_dialog_add_button(GtkDialog *dlg, const char *name, const char *text,
						   gint id, gboolean def_resp)
{
	GtkWidget *btn, *img;

	btn = gtk_dialog_add_button(dlg, text, id);
	if(GTK_RESPONSE_OK == def_resp || GTK_RESPONSE_CANCEL == def_resp ||
	   GTK_RESPONSE_YES == def_resp || GTK_RESPONSE_CLOSE == def_resp ||
	   GTK_RESPONSE_ACCEPT == def_resp){
		img = image_from_default_theme(name, GTK_ICON_SIZE_BUTTON);
	}
	else
		img = gtk_image_new_from_icon_name(name, GTK_ICON_SIZE_BUTTON);

	gtk_button_set_image(GTK_BUTTON(btn), img);

	if(def_resp)
		gtk_dialog_set_default_response(dlg, id);
}

#if GTK_CHECK_VERSION(3, 12, 0)
void util_set_select_dlg_button_image(GtkDialog *dialog, int resp_id,
								 const char *name)
{
	GtkWidget *button, *image;

	button = gtk_dialog_get_widget_for_response(dialog, resp_id);
	if(GTK_IS_BUTTON(button)){
		image = gtk_image_new_from_icon_name(name, GTK_ICON_SIZE_BUTTON);
		g_object_set(button, "image", image, NULL);
		gtk_widget_show(image);
	}
}
#endif
